@@ -9,7 +9,7 @@
 
   echo "[encoding: UTF-8]" > po/POTFILES.in \
   && ls -1 data/gajim.desktop.in.in data/glade/*.glade \
-  src/*py src/common/*py src/common/zeroconf/*.py src/osx/*.py >> \
+  src/*py src/common/*py src/common/zeroconf/*.py | grep -v ipython_view.py >> \
   po/POTFILES.in || exit 1
   if test -z `which pkg-config 2>/dev/null`;then
     echo "***Error: pkg-config not found***"
