@@ -1,17 +1,26 @@
 # -*- coding: utf-8 -*-
-##	adhoc_commands.py
+## src/adhoc_commands.py
 ##
-## Copyright (C) 2006 Yann Le Boulanger <asterix@lagaule.org>
-##                    Nikos Kouremenos <nkour@jabber.org>
+## Copyright (C) 2006 Nikos Kouremenos <kourem AT gmail.com>
+## Copyright (C) 2006-2007 Tomasz Melcer <liori AT exroot.org>
+## Copyright (C) 2006-2008 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2008 Jonathan Schleifer <js-gajim AT webkeks.org>
+##                    Stephan Erb <steve-e AT h3c.de>
 ##
-## This program is free software; you can redistribute it and/or modify
+## This file is part of Gajim.
+##
+## Gajim is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 2 only.
+## by the Free Software Foundation; version 3 only.
 ##
-## This program is distributed in the hope that it will be useful,
+## Gajim is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ## GNU General Public License for more details.
+##
+## You should have received a copy of the GNU General Public License
+## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+##
 
 # FIXME: think if we need caching command list. it may be wrong if there will
 # be entities that often change the list, it may be slow to fetch it every time
@@ -35,7 +44,7 @@ class CommandWindow:
 
 	TODO: gtk 2.10 has a special wizard-widget, consider using it...'''
 
-	def __init__(self, account, jid):
+	def __init__(self, account, jid, commandnode=None):
 		'''Create new window.'''
 
 		# an account object
@@ -46,9 +55,10 @@ class CommandWindow:
 		self.commandlist=None	# a list of (commandname, commanddescription)
 
 		# command's data
-		self.commandnode = None
+		self.commandnode = commandnode
 		self.sessionid = None
 		self.dataform = None
+		self.allow_stage3_close = False
 
 		# retrieving widgets from xml
 		self.xml = gtkgui_helpers.get_glade('adhoc_commands_window.glade')
@@ -68,8 +78,12 @@ class CommandWindow:
 		self.data_form_widget.show()
 		self.sending_form_stage_vbox.pack_start(self.data_form_widget)
 
-		# setting initial stage
-		self.stage1()
+		if self.commandnode:
+			# Execute command
+			self.stage3()
+		else:
+			# setting initial stage
+			self.stage1()
 
 		# displaying the window
 		self.xml.signal_autoconnect(self)
@@ -105,7 +119,7 @@ class CommandWindow:
 		return self.stage_adhoc_commands_window_delete_event(self.window)
 
 	def __del__(self):
-		print "Object has been deleted."
+		print 'Object has been deleted.'
 
 # stage 1: waiting for command list
 	def stage1(self):
@@ -172,7 +186,7 @@ class CommandWindow:
 		for (commandnode, commandname) in self.commandlist:
 			radio = gtk.RadioButton(first_radio, label=commandname)
 			radio.connect("toggled", self.on_command_radiobutton_toggled, commandnode)
-			if first_radio is None:
+			if not first_radio:
 				first_radio = radio
 				self.commandnode = commandnode
 			self.command_list_vbox.pack_start(radio, expand=False)
@@ -242,18 +256,20 @@ class CommandWindow:
 				self.window.destroy()
 			return False
 
-		dialog = dialogs.HigDialog(self.window, gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_MODAL,
-			gtk.BUTTONS_YES_NO, 'Cancel confirmation',
-			'You are in process of executing command. Do you really want to cancel it?')
-		dialog.popup()
-		if dialog.get_response()==gtk.RESPONSE_YES:
-			self.send_cancel()
-			if widget==self.window:
-				return False
-			else:
-				self.window.destroy()
+		if self.allow_stage3_close:
 			return False
-		return True
+
+		def on_yes(button):
+			self.send_cancel()
+			self.allow_stage3_close = True
+			self.window.destroy()
+
+		dialog = dialogs.HigDialog(self.window, gtk.DIALOG_DESTROY_WITH_PARENT | \
+			gtk.DIALOG_MODAL, gtk.BUTTONS_YES_NO, _('Cancel confirmation'),
+			_('You are in process of executing command. Do you really want to '
+			'cancel it?'), on_response_yes=on_yes)
+		dialog.popup()
+		return True # Block event, don't close window
 
 	def stage3_back_button_clicked(self, widget):
 		self.stage3_submit_form('prev')
@@ -266,10 +282,10 @@ class CommandWindow:
 
 	def stage3_submit_form(self, action='execute'):
 		self.data_form_widget.set_sensitive(False)
-		if self.data_form_widget.get_data_form() is None:
-			self.data_form_widget.hide()
-		else:
+		if self.data_form_widget.get_data_form():
 			self.data_form_widget.data_form.type='submit'
+		else:
+			self.data_form_widget.hide()
 
 		self.close_button.set_sensitive(True)
 		self.back_button.set_sensitive(False)
@@ -286,40 +302,43 @@ class CommandWindow:
 		self.remove_pulsing()
 		self.sending_form_progressbar.hide()
 
-		self.sessionid = command.getAttr('sessionid')
+		if not self.sessionid:
+			self.sessionid = command.getAttr('sessionid')
+		elif self.sessionid != command.getAttr('sessionid'):
+			self.stage5(error=_('Service changed the session identifier.'),
+				senderror=True)
 
 		self.form_status = command.getAttr('status')
 
 		self.commandnode = command.getAttr('node')
-		if command.getTag('x') is not None:
+		if command.getTag('x'):
 			self.dataform = dataforms.ExtendForm(node=command.getTag('x'))
 
 			self.data_form_widget.set_sensitive(True)
 			try:
 				self.data_form_widget.data_form=self.dataform
 			except dataforms.Error:
-				# FIXME: translate
-				self.stage5(error='Service sent malformed data', senderror=True)
+				self.stage5(error=_('Service sent malformed data'), senderror=True)
 				return
 			self.data_form_widget.show()
 			if self.data_form_widget.title:
-				self.window.set_title("%s - Ad-hoc Commands - Gajim" % \
+				self.window.set_title('%s - Ad-hoc Commands - Gajim' % \
 					self.data_form_widget.title)
 		else:
 			self.data_form_widget.hide()
 
 		actions = command.getTag('actions')
-		if actions is None:
-			self.close_button.set_sensitive(True)
-			self.back_button.set_sensitive(False)
-			self.forward_button.set_sensitive(False)
-			self.execute_button.set_sensitive(True)
-		else:
+		if actions:
 			# actions, actions, actions...
 			self.close_button.set_sensitive(True)
 			self.back_button.set_sensitive(actions.getTag('prev') is not None)
 			self.forward_button.set_sensitive(actions.getTag('next') is not None)
 			self.execute_button.set_sensitive(True)
+		else:
+			self.close_button.set_sensitive(True)
+			self.back_button.set_sensitive(False)
+			self.forward_button.set_sensitive(False)
+			self.execute_button.set_sensitive(True)
 
 		if self.form_status == 'completed':
 			self.close_button.set_sensitive(True)
@@ -330,7 +349,7 @@ class CommandWindow:
 			self.stage_adhoc_commands_window_delete_event = self.stage3_close_button_clicked
 
 		note = command.getTag('note')
-		if note is not None:
+		if note:
 			self.notes_label.set_text(note.getData().decode('utf-8'))
 			self.notes_label.set_no_show_all(False)
 			self.notes_label.show()
@@ -370,9 +389,9 @@ class CommandWindow:
 		# close old stage
 		self.stage_finish()
 
-		assert errorid is not None or error is not None
+		assert errorid or error
 
-		if errorid is not None:
+		if errorid:
 			# we've got error code, display appropriate message
 			try:
 				errorname = xmpp.NS_STANZAS + ' ' + str(errorid)
@@ -380,8 +399,8 @@ class CommandWindow:
 				error = errordesc.decode('utf-8')
 				del errorname, errordesc
 			except KeyError:	# when stanza doesn't have error description
-				error = 'Service returned an error.'
-		elif error is not None:
+				error = _('Service returned an error.')
+		elif error:
 			# we've got error message
 			pass
 		else:
@@ -410,7 +429,7 @@ class CommandWindow:
 	def setup_pulsing(self, progressbar):
 		'''Set the progressbar to pulse. Makes a custom
 		function to repeatedly call progressbar.pulse() method.'''
-		assert self.pulse_id is None
+		assert not self.pulse_id
 		assert isinstance(progressbar, gtk.ProgressBar)
 
 		def callback():
@@ -422,14 +441,14 @@ class CommandWindow:
 
 	def remove_pulsing(self):
 		'''Stop pulsing, useful when especially when removing widget.'''
-		if self.pulse_id is not None:
+		if self.pulse_id:
 			gobject.source_remove(self.pulse_id)
 		self.pulse_id=None
 
 # handling xml stanzas
 	def request_command_list(self):
 		'''Request the command list. Change stage on delivery.'''
-		query = xmpp.Iq(typ='get', to=xmpp.JID(self.jid), xmlns=xmpp.NS_DISCO_ITEMS)
+		query = xmpp.Iq(typ='get', to=xmpp.JID(self.jid), queryNS=xmpp.NS_DISCO_ITEMS)
 		query.setQuerynode(xmpp.NS_COMMANDS)
 
 		def callback(response):
@@ -437,7 +456,7 @@ class CommandWindow:
 			# FIXME: move to connection_handlers.py
 			# is error => error stage
 			error = response.getError()
-			if error is not None:
+			if error:
 				# extracting error description from xmpp/protocol.py
 				self.stage5(errorid = error)
 				return
@@ -471,10 +490,10 @@ class CommandWindow:
 				'action':action
 			})
 
-		if self.sessionid is not None:
+		if self.sessionid:
 			cmdnode.setAttr('sessionid', self.sessionid)
 
-		if self.data_form_widget.data_form is not None:
+		if self.data_form_widget.data_form:
 #			cmdnode.addChild(node=dataforms.DataForm(tofill=self.data_form_widget.data_form))
 			# FIXME: simplified form to send
 			
@@ -483,7 +502,7 @@ class CommandWindow:
 		def callback(response):
 			# FIXME: move to connection_handlers.py
 			err = response.getError()
-			if err is not None:
+			if err:
 				self.stage5(errorid = err)
 			else:
 				self.stage3_next_form(response.getTag('command'))
@@ -492,8 +511,8 @@ class CommandWindow:
 
 	def send_cancel(self):
 		'''Send the command with action='cancel'. '''
-		assert self.commandnode is not None
-		if self.sessionid is not None and self.account.connection:
+		assert self.commandnode
+		if self.sessionid and self.account.connection:
 			# we already have sessionid, so the service sent at least one reply.
 			stanza = xmpp.Iq(typ='set', to=self.jid)
 			stanza.addChild('command', attrs={
@@ -508,3 +527,5 @@ class CommandWindow:
 			# we did not received any reply from service; FIXME: we should wait and
 			# then send cancel; for now we do nothing
 			pass
+
+# vim: se ts=3:
