@@ -1,26 +1,27 @@
-## Contributors for this file:
-## - Yann Le Boulanger <asterix@lagaule.org>
-## - Nikos Kouremenos <kourem@gmail.com>
-## - Travis Shirk <travis@pobox.com>
+# -*- coding:utf-8 -*-
+## src/common/check_paths.py
 ##
-## Copyright (C) 2003-2004 Yann Le Boulanger <asterix@lagaule.org>
-##                         Vincent Hanquez <tab@snarc.org>
-## Copyright (C) 2005 Yann Le Boulanger <asterix@lagaule.org>
-##                    Vincent Hanquez <tab@snarc.org>
-##                    Nikos Kouremenos <nkour@jabber.org>
-##                    Dimitur Kirov <dkirov@gmail.com>
-##                    Travis Shirk <travis@pobox.com>
-##                    Norman Rasmussen <norman@rasmussen.co.za>
+## Copyright (C) 2005-2006 Travis Shirk <travis AT pobox.com>
+##                         Nikos Kouremenos <kourem AT gmail.com>
+## Copyright (C) 2005-2008 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2006 Dimitur Kirov <dkirov AT gmail.com>
+## Copyright (C) 2007 Tomasz Melcer <liori AT exroot.org>
+## Copyright (C) 2008 Jean-Marie Traissard <jim AT lapin.org>
 ##
-## This program is free software; you can redistribute it and/or modify
+## This file is part of Gajim.
+##
+## Gajim is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 2 only.
+## by the Free Software Foundation; version 3 only.
 ##
-## This program is distributed in the hope that it will be useful,
+## Gajim is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ## GNU General Public License for more details.
 ##
+## You should have received a copy of the GNU General Public License
+## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+##
 
 import os
 import sys
@@ -28,12 +29,15 @@ import stat
 
 from common import gajim
 import logger
-import i18n
 
-_ = i18n._
-Q_ = i18n.Q_
-
-from pysqlite2 import dbapi2 as sqlite # DO NOT MOVE ABOVE OF import gajim
+# DO NOT MOVE ABOVE OF import gajim
+try:
+	import sqlite3 as sqlite # python 2.5
+except ImportError:
+	try:
+		from pysqlite2 import dbapi2 as sqlite
+	except ImportError:
+		raise exceptions.PysqliteNotAvailable
 
 def create_log_db():
 	print _('creating logs database')
@@ -48,6 +52,7 @@ def create_log_db():
 	# logs.jid_id --> jids.jid_id but Sqlite doesn't do FK etc so it's done in python code
 	# jids.jid text column will be JID if TC-related, room_jid if GC-related,
 	# ROOM_JID/nick if pm-related.
+	# also check optparser.py, which updates databases on gajim updates
 	cur.executescript(
 		'''
 		CREATE TABLE jids(
@@ -61,6 +66,13 @@ def create_log_db():
 			jid_id INTEGER
 		);
 		
+		CREATE INDEX idx_unread_messages_jid_id ON unread_messages (jid_id);
+		
+		CREATE TABLE transports_cache (
+			transport TEXT UNIQUE,
+			type INTEGER
+		);
+		
 		CREATE TABLE logs(
 			log_line_id INTEGER PRIMARY KEY AUTOINCREMENT UNIQUE,
 			jid_id INTEGER,
@@ -71,6 +83,18 @@ def create_log_db():
 			message TEXT,
 			subject TEXT
 		);
+		
+		CREATE INDEX idx_logs_jid_id_kind ON logs (jid_id, kind);
+
+		CREATE TABLE caps_cache (
+			hash_method TEXT,
+			hash TEXT,
+			data BLOB);
+
+		CREATE TABLE IF NOT EXISTS rooms_last_message_time(
+			jid_id INTEGER PRIMARY KEY UNIQUE,
+			time INTEGER
+		);
 		'''
 		)
 
@@ -83,7 +107,7 @@ def check_and_possibly_create_paths():
 	AVATAR_PATH = gajim.AVATAR_PATH
 	dot_gajim = os.path.dirname(VCARD_PATH)
 	if os.path.isfile(dot_gajim):
-		print _('%s is file but it should be a directory') % dot_gajim
+		print _('%s is a file but it should be a directory') % dot_gajim
 		print _('Gajim will now exit')
 		sys.exit()
 	elif os.path.isdir(dot_gajim):
@@ -94,14 +118,14 @@ def check_and_possibly_create_paths():
 		if not os.path.exists(VCARD_PATH):
 			create_path(VCARD_PATH)
 		elif os.path.isfile(VCARD_PATH):
-			print _('%s is file but it should be a directory') % VCARD_PATH
+			print _('%s is a file but it should be a directory') % VCARD_PATH
 			print _('Gajim will now exit')
 			sys.exit()
 
 		if not os.path.exists(AVATAR_PATH):
 			create_path(AVATAR_PATH)
 		elif os.path.isfile(AVATAR_PATH):
-			print _('%s is file but it should be a directory') % AVATAR_PATH
+			print _('%s is a file but it should be a directory') % AVATAR_PATH
 			print _('Gajim will now exit')
 			sys.exit()
 
@@ -109,10 +133,10 @@ def check_and_possibly_create_paths():
 			create_log_db()
 			gajim.logger.init_vars()
 		elif os.path.isdir(LOG_DB_PATH):
-			print _('%s is directory but should be file') % LOG_DB_PATH
+			print _('%s is a directory but should be a file') % LOG_DB_PATH
 			print _('Gajim will now exit')
 			sys.exit()
-
+		
 	else: # dot_gajim doesn't exist
 		if dot_gajim: # is '' on win9x so avoid that
 			create_path(dot_gajim)
@@ -127,3 +151,5 @@ def check_and_possibly_create_paths():
 def create_path(directory):
 	print _('creating %s directory') % directory
 	os.mkdir(directory, 0700)
+
+# vim: se ts=3:
