@@ -55,8 +55,9 @@ if dbus_support.supported:
 	import dbus
 	from music_track_listener import MusicTrackListener
 
-STATUS_LIST = ['offline', 'connecting', 'online', 'chat', 'away', 'xa', 'dnd',
-	'invisible', 'error']
+import logging
+log = logging.getLogger('gajim.c.connection_handlers')
+
 # kind of events we can wait for an answer
 VCARD_PUBLISHED = 'vcard_published'
 VCARD_ARRIVED = 'vcard_arrived'
@@ -108,13 +109,13 @@ class ConnectionBytestream:
 		for file_props in self.files_props.values():
 			if self.is_transfer_stopped(file_props):
 				continue
-			receiver_jid = unicode(file_props['receiver']).split('/')[0]
-			if contact.jid == receiver_jid:
+			receiver_jid = unicode(file_props['receiver'])
+			if contact.get_full_jid() == receiver_jid:
 				file_props['error'] = -5
 				self.remove_transfer(file_props)
 				self.dispatch('FILE_REQUEST_ERROR', (contact.jid, file_props, ''))
-			sender_jid = unicode(file_props['sender']).split('/')[0]
-			if contact.jid == sender_jid:
+			sender_jid = unicode(file_props['sender'])
+			if contact.get_full_jid() == sender_jid:
 				file_props['error'] = -3
 				self.remove_transfer(file_props)
 
@@ -223,12 +224,18 @@ class ConnectionBytestream:
 			ostreamhost.setAttr('host', ft_host)
 			ostreamhost.setAttr('jid', sender)
 		try:
-			thehost = self.peerhost[0]
-			streamhost = common.xmpp.Node(tag = 'streamhost') # My IP
-			query.addChild(node = streamhost)
-			streamhost.setAttr('port', unicode(port))
-			streamhost.setAttr('host', thehost)
-			streamhost.setAttr('jid', sender)
+			# The ip we're connected to server with
+			my_ips = [self.peerhost[0]]
+			# all IPs from local DNS
+			for addr in socket.getaddrinfo(socket.gethostname(), None):
+				if not addr[4][0] in my_ips:
+					my_ips.append(addr[4][0])
+			for ip in my_ips:
+				streamhost = common.xmpp.Node(tag = 'streamhost')
+				query.addChild(node = streamhost)
+				streamhost.setAttr('port', unicode(port))
+				streamhost.setAttr('host', ip)
+				streamhost.setAttr('jid', sender)
 		except socket.gaierror:
 			self.dispatch('ERROR', (_('Wrong host'),
 				_('Invalid local address? :-O')))
@@ -483,7 +490,7 @@ class ConnectionBytestream:
 						gajim.socks5queue.activate_proxy(host['idx'])
 						break
 			raise common.xmpp.NodeProcessed
-		jid = streamhost.getAttr('jid')
+		jid = helpers.parse_jid(streamhost.getAttr('jid'))
 		if 'streamhost-used' in file_props and \
 			file_props['streamhost-used'] is True:
 			raise common.xmpp.NodeProcessed
@@ -631,16 +638,26 @@ class ConnectionDisco:
 	def request_register_agent_info(self, agent):
 		if not self.connection or self.connected < 2:
 			return None
-		iq=common.xmpp.Iq('get', common.xmpp.NS_REGISTER, to=agent)
+		iq = common.xmpp.Iq('get', common.xmpp.NS_REGISTER, to=agent)
 		id = self.connection.getAnID()
 		iq.setID(id)
 		# Wait the answer during 30 secondes
 		self.awaiting_timeouts[gajim.idlequeue.current_time() + 30] = (id,
-			_('Registration information for transport %s has not arrived in time') % \
-			agent)
+			_('Registration information for transport %s has not arrived in time')\
+			% agent)
 		self.connection.SendAndCallForResponse(iq, self._ReceivedRegInfo,
 			{'agent': agent})
 
+	def _agent_registered_cb(self, con, resp, agent):
+		if resp.getType() == 'result':
+			self.dispatch('INFORMATION', (_('Registration succeeded'),
+				_('Resgitration with agent %s succeeded') % agent))
+		if resp.getType() == 'error':
+			self.dispatch('ERROR', (_('Registration failed'), _('Resgitration with'
+				' agent %(agent)s failed with error %(error)s: %(error_msg)s') % {
+				'agent': agent, 'error': resp.getError(),
+				'error_msg': resp.getErrorMsg()}))
+
 	def register_agent(self, agent, info, is_form = False):
 		if not self.connection or self.connected < 2:
 			return
@@ -649,7 +666,8 @@ class ConnectionDisco:
 			query = iq.getTag('query')
 			info.setAttr('type', 'submit')
 			query.addChild(node = info)
-			self.connection.send(iq)
+			self.connection.SendAndCallForResponse(iq, self._agent_registered_cb,
+				{'agent': agent})
 		else:
 			# fixed: blocking
 			common.xmpp.features_nb.register(self.connection, agent, info, None)
@@ -716,7 +734,7 @@ class ConnectionDisco:
 			if 'jid' not in attr:
 				continue
 			try:
-				helpers.parse_jid(attr['jid'])
+				attr['jid'] = helpers.parse_jid(attr['jid'])
 			except common.helpers.InvalidFormat:
 				# jid is not conform
 				continue
@@ -725,9 +743,9 @@ class ConnectionDisco:
 		hostname = gajim.config.get_per('accounts', self.name,
 													'hostname')
 		id = iq_obj.getID()
-		if jid == hostname and id[0] == 'p':
+		if jid == hostname and id[:6] == 'Gajim_':
 			for item in items:
-				self.discoverInfo(item['jid'], id_prefix='p')
+				self.discoverInfo(item['jid'], id_prefix='Gajim_')
 		else:
 			self.dispatch('AGENT_INFO_ITEMS', (jid, node, items))
 
@@ -759,7 +777,7 @@ class ConnectionDisco:
 			raise common.xmpp.NodeProcessed
 
 		id = unicode(iq_obj.getAttr('id'))
-		if id[0] == 'p':
+		if id[:6] == 'Gajim_':
 			# We get this request from echo.server
 			raise common.xmpp.NodeProcessed
 
@@ -790,6 +808,8 @@ class ConnectionDisco:
 
 	def _DiscoverInfoCB(self, con, iq_obj):
 		gajim.log.debug('DiscoverInfoCB')
+		if not self.connection or self.connected < 2:
+			return
 		# According to XEP-0030:
 		# For identity: category, type is mandatory, name is optional.
 		# For feature: var is mandatory
@@ -826,10 +846,13 @@ class ConnectionDisco:
 			gajim.transport_type[jid] = transport_type
 			gajim.logger.save_transport_type(jid, transport_type)
 		id = iq_obj.getID()
+		if id is None:
+			log.warn('Invalid IQ received without an ID. Ignoring it: %s' % iq_obj)
+			return
 		if not identities: # ejabberd doesn't send identities when we browse online users
 		#FIXME: see http://www.jabber.ru/bugzilla/show_bug.cgi?id=225
 			identities = [{'category': 'server', 'type': 'im', 'name': node}]
-		if id[0] == 'p':
+		if id[:6] == 'Gajim_':
 			if jid == gajim.config.get_per('accounts', self.name, 'hostname'):
 				if features.__contains__(common.xmpp.NS_GMAILNOTIFY):
 					gajim.gmail_domains.append(jid)
@@ -1056,12 +1079,12 @@ class ConnectionVcard:
 				self.save_vcard_to_hd(our_jid, vcard_iq)
 
 				# Send new presence if sha changed and we are not invisible
-				if self.vcard_sha != new_sha and STATUS_LIST[self.connected] != \
+				if self.vcard_sha != new_sha and gajim.SHOW_LIST[self.connected] !=\
 				'invisible':
 					if not self.connection or self.connected < 2:
 						return
 					self.vcard_sha = new_sha
-					sshow = helpers.get_xmpp_show(STATUS_LIST[self.connected])
+					sshow = helpers.get_xmpp_show(gajim.SHOW_LIST[self.connected])
 					p = common.xmpp.Presence(typ = None, priority = self.priority,
 						show = sshow, status = self.status)
 					p = self.add_sha(p)
@@ -1105,7 +1128,10 @@ class ConnectionVcard:
 				storage = query.getTag('storage')
 				metas = storage.getTags('meta')
 				for meta in metas:
-					jid = meta.getAttr('jid')
+					try:
+						jid = helpers.parse_jid(meta.getAttr('jid'))
+					except common.helpers.InvalidFormat:
+						continue
 					tag = meta.getAttr('tag')
 					data = {'jid': jid}
 					order = meta.getAttr('order')
@@ -1228,11 +1254,11 @@ class ConnectionVcard:
 			if self.vcard_sha == avatar_sha:
 				return
 			self.vcard_sha = avatar_sha
-			if STATUS_LIST[self.connected] == 'invisible':
+			if gajim.SHOW_LIST[self.connected] == 'invisible':
 				return
 			if not self.connection:
 				return
-			sshow = helpers.get_xmpp_show(STATUS_LIST[self.connected])
+			sshow = helpers.get_xmpp_show(gajim.SHOW_LIST[self.connected])
 			p = common.xmpp.Presence(typ = None, priority = self.priority,
 				show = sshow, status = self.status)
 			p = self.add_sha(p)
@@ -1465,14 +1491,18 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 					print_status = conf.getTagData('print_status')
 					if not print_status:
 						print_status = conf.getTagData('show_status')
-					bm = {'name': conf.getAttr('name'),
-							'jid': conf.getAttr('jid'),
+					try:
+						bm = {'name': conf.getAttr('name'),
+							'jid': helpers.parse_jid(conf.getAttr('jid')),
 							'autojoin': autojoin_val,
 							'minimize': minimize_val,
 							'password': conf.getTagData('password'),
 							'nick': conf.getTagData('nick'),
 							'print_status': print_status}
-					
+					except common.helpers.InvalidFormat:
+						log.warn('Invalid JID: %s, ignoring it' % conf.getAttr('jid'))
+						continue
+
 					self.bookmarks.append(bm)
 				self.dispatch('BOOKMARKS', self.bookmarks)
 
@@ -1486,14 +1516,22 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 				# http://www.xmpp.org/extensions/xep-0145.html
 				notes = storage.getTags('note')
 				for note in notes:
-					jid = note.getAttr('jid')
+					try:
+						jid = helpers.parse_jid(note.getAttr('jid'))
+					except common.helpers.InvalidFormat:
+						log.warn('Invalid JID: %s, ignoring it' % note.getAttr('jid'))
+						continue
 					annotation = note.getData()
 					self.annotations[jid] = annotation
 
 	def _rosterSetCB(self, con, iq_obj):
 		gajim.log.debug('rosterSetCB')
 		for item in iq_obj.getTag('query').getChildren():
-			jid = helpers.parse_jid(item.getAttr('jid'))
+			try:
+				jid = helpers.parse_jid(item.getAttr('jid'))
+			except common.helpers.InvalidFormat:
+				log.warn('Invalid JID: %s, ignoring it' % item.getAttr('jid'))
+				continue
 			name = item.getAttr('name')
 			sub = item.getAttr('subscription')
 			ask = item.getAttr('ask')
@@ -1501,6 +1539,11 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 			for group in item.getTags('group'):
 				groups.append(group.getData())
 			self.dispatch('ROSTER_INFO', (jid, name, sub, ask, groups))
+		if not self.connection or self.connected < 2:
+			raise common.xmpp.NodeProcessed
+		reply = common.xmpp.Iq(typ='result', attrs={'id': iq_obj.getID()},
+			to=iq_obj.getFrom(), frm=iq_obj.getTo(), xmlns=None)
+		self.connection.send(reply)
 		raise common.xmpp.NodeProcessed
 	
 	def _VersionCB(self, con, iq_obj):
@@ -1511,7 +1554,7 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 		qp = iq_obj.getTag('query')
 		qp.setTagData('name', 'Gajim')
 		qp.setTagData('version', gajim.version)
-		send_os = gajim.config.get('send_os_info')
+		send_os = gajim.config.get_per('accounts', self.name, 'send_os_info')
 		if send_os:
 			qp.setTagData('os', helpers.get_os_info())
 		self.connection.send(iq_obj)
@@ -1594,7 +1637,8 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 		qp = iq_obj.setTag('time',
 			namespace=common.xmpp.NS_TIME_REVISED)
 		qp.setTagData('utc', strftime('%Y-%m-%dT%H:%M:%SZ', gmtime()))
-		zone = -(timezone, altzone)[daylight] / 60
+		isdst = localtime().tm_isdst
+		zone = -(timezone, altzone)[isdst] / 60
 		tzo = (zone / 60, abs(zone % 60))
 		qp.setTagData('tzo', '%+03d:%02d' % (tzo))
 		self.connection.send(iq_obj)
@@ -1672,8 +1716,11 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 		'''Called when we receive a message'''
 		gajim.log.debug('MessageCB')
 
+		mtype = msg.getType()
 		# check if the message is pubsub#event
 		if msg.getTag('event') is not None:
+			if mtype == 'groupchat':
+				return
 			if msg.getTag('error') is None:
 				self._pubsubEventCB(con, msg)
 			return
@@ -1692,7 +1739,11 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 		if addressTag and jid == gajim.get_jid_from_account(self.name):
 			address = addressTag.getTag('address', attrs={'type': 'ofrom'})
 			if address:
-				frm = address.getAttr('jid')
+				try:
+					frm = helpers.parse_jid(address.getAttr('jid'))
+				except common.helpers.InvalidFormat:
+					log.warn('Invalid JID: %s, ignoring it' % address.getAttr('jid'))
+					return
 				jid = gajim.get_jid_without_resource(frm)
 
 		# invitations
@@ -1710,7 +1761,11 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 		xtags = msg.getTags('x')
 		for xtag in xtags:
 			if xtag.getNamespace() == common.xmpp.NS_CONFERENCE and not invite:
-				room_jid = xtag.getAttr('jid')
+				try:
+					room_jid = helpers.parse_jid(xtag.getAttr('jid'))
+				except common.helpers.InvalidFormat:
+					log.warn('Invalid JID: %s, ignoring it' % xtag.getAttr('jid'))
+					continue
 				is_continued = False
 				if xtag.getTag('continue'):
 					is_continued = True
@@ -1718,7 +1773,6 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 					is_continued))
 				return
 
-		mtype = msg.getType()
 		thread_id = msg.getThread()
 
 		if not mtype:
@@ -1812,7 +1866,7 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 			if keyID:
 				decmsg = self.gpg.decrypt(encmsg, keyID)
 				# \x00 chars are not allowed in C (so in GTK)
-				msgtxt = decmsg.replace('\x00', '')
+				msgtxt = helpers.decode_string(decmsg.replace('\x00', ''))
 				encrypted = 'xep27'
 		if mtype == 'error':
 			self.dispatch_error_message(msg, msgtxt, session, frm, tim)
@@ -1898,7 +1952,11 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 
 	def dispatch_invite_message(self, invite, frm):
 		item = invite.getTag('invite')
-		jid_from = item.getAttr('from')
+		try:
+			jid_from = helpers.parse_jid(item.getAttr('from'))
+		except common.helpers.InvalidFormat:
+			log.warn('Invalid JID: %s, ignoring it' % item.getAttr('from'))
+			return
 		reason = item.getTagData('reason')
 		item = invite.getTag('password')
 		password = invite.getTagData('password')
@@ -1954,10 +2012,13 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 		if ptype and not ptype in rfc_types:
 			ptype = None
 		gajim.log.debug('PresenceCB: %s' % ptype)
+		if not self.connection or self.connected < 2:
+			log.debug('account is no more connected')
+			return
 		try:
 			who = helpers.get_full_jid_from_iq(prs)
 		except Exception:
-			if prs.getTag('error').getTag('jid-malformed'):
+			if prs.getTag('error') and prs.getTag('error').getTag('jid-malformed'):
 				# wrong jid, we probably tried to change our nick in a room to a non
 				# valid one
 				who = str(prs.getFrom())
@@ -2099,7 +2160,10 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 					r = destroy.getTagData('reason')
 					if r:
 						reason += ' (%s)' % r
-					jid = destroy.getAttr('jid')
+					try:
+						jid = helpers.parse_jid(destroy.getAttr('jid'))
+					except common.helpers.InvalidFormat:
+						pass
 					if jid:
 						reason += '\n' + _('You can join this room instead: %s') % jid
 					statusCode = ['destroyed']
@@ -2175,8 +2239,9 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 				# print in the window the error
 				self.dispatch('ERROR_ANSWER', ('', jid_stripped,
 					errmsg, errcode))
-			self.dispatch('NOTIFY', (jid_stripped, 'error', errmsg, resource, prio,
-				keyID, timestamp, None))
+			if errcode != '409': # conflict # See #5120
+				self.dispatch('NOTIFY', (jid_stripped, 'error', errmsg, resource,
+					prio, keyID, timestamp, None))
 
 		if ptype == 'unavailable' and jid_stripped in self.sessions:
 			# automatically terminate sessions that they haven't sent a thread ID
@@ -2246,7 +2311,11 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 		users_dict = {}
 		for item in items:
 			if item.has_attr('jid') and item.has_attr('affiliation'):
-				jid = item.getAttr('jid')
+				try:
+					jid = helpers.parse_jid(item.getAttr('jid'))
+				except common.helpers.InvalidFormat:
+					log.warn('Invalid JID: %s, ignoring it' % item.getAttr('jid'))
+					return
 				affiliation = item.getAttr('affiliation')
 				users_dict[jid] = {'affiliation': affiliation}
 				if item.has_attr('nick'):
@@ -2292,13 +2361,14 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 		raise common.xmpp.NodeProcessed
 
 	def _getRosterCB(self, con, iq_obj):
+		log.debug('getRosterCB')
 		if not self.connection:
 			return
 		self.connection.getRoster(self._on_roster_set)
 		self.discoverItems(gajim.config.get_per('accounts', self.name, 
-			'hostname'), id_prefix='p')
+			'hostname'), id_prefix='Gajim_')
 		self.discoverInfo(gajim.config.get_per('accounts', self.name, 
-			'hostname'), id_prefix='p')
+			'hostname'), id_prefix='Gajim_')
 		if gajim.config.get_per('accounts', self.name, 'use_ft_proxies'):
 			self.discover_ft_proxies()
 	
@@ -2365,7 +2435,7 @@ class ConnectionHandlers(ConnectionVcard, ConnectionBytestream, ConnectionDisco,
 					_('You will be connected to %s without OpenPGP.') % self.name))
 				self.USE_GPG = False
 				signed = ''
-		self.connected = STATUS_LIST.index(show)
+		self.connected = gajim.SHOW_LIST.index(show)
 		sshow = helpers.get_xmpp_show(show)
 		# send our presence
 		if show == 'invisible':
