@@ -1,26 +1,42 @@
-## common/contacts.py
+# -*- coding:utf-8 -*-
+## src/common/contacts.py
 ##
-## Copyright (C) 2006 Yann Le Boulanger <asterix@lagaule.org>
-## Copyright (C) 2006 Nikos Kouremenos <kourem@gmail.com>
+## Copyright (C) 2006 Dimitur Kirov <dkirov AT gmail.com>
+##                    Travis Shirk <travis AT pobox.com>
+##                    Nikos Kouremenos <kourem AT gmail.com>
+## Copyright (C) 2006-2008 Yann Leboulanger <asterix AT lagaule.org>
+##                         Jean-Marie Traissard <jim AT lapin.org>
+## Copyright (C) 2007 Lukas Petrovicky <lukas AT petrovicky.net>
+##                    Tomasz Melcer <liori AT exroot.org>
+##                    Julien Pivotto <roidelapluie AT gmail.com>
+## Copyright (C) 2007-2008 Stephan Erb <steve-e AT h3c.de>
+## Copyright (C) 2008 Brendan Taylor <whateley AT gmail.com>
+##                    Jonathan Schleifer <js-gajim AT webkeks.org>
 ##
+## This file is part of Gajim.
 ##
-## This program is free software; you can redistribute it and/or modify
+## Gajim is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 2 only.
+## by the Free Software Foundation; version 3 only.
 ##
-## This program is distributed in the hope that it will be useful,
+## Gajim is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ## GNU General Public License for more details.
 ##
+## You should have received a copy of the GNU General Public License
+## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+##
 
 import common.gajim
 
 class Contact:
 	'''Information concerning each contact'''
 	def __init__(self, jid='', name='', groups=[], show='', status='', sub='',
-	ask='', resource='', priority=0, keyID='', our_chatstate=None,
-	chatstate=None, last_status_time=None, msg_id = None, composing_xep = None):
+	ask='', resource='', priority=0, keyID='', caps_node=None,
+	caps_hash_method=None, caps_hash=None, our_chatstate=None, chatstate=None,
+	last_status_time=None, msg_id = None, composing_xep = None,	mood={}, tune={},
+	activity={}):
 		self.jid = jid
 		self.name = name
 		self.contact_name = '' # nick choosen by contact
@@ -33,22 +49,31 @@ class Contact:
 		self.priority = priority
 		self.keyID = keyID
 
-		# please read jep-85 http://www.jabber.org/jeps/jep-0085.html
-		# we keep track of jep85 support with the peer by three extra states:
+		# Capabilities; filled by caps.py/ConnectionCaps object
+		# every time it gets these from presence stanzas
+		self.caps_node = caps_node
+		self.caps_hash_method = caps_hash_method
+		self.caps_hash = caps_hash
+
+		# please read xep-85 http://www.xmpp.org/extensions/xep-0085.html
+		# we keep track of xep85 support with the peer by three extra states:
 		# None, False and 'ask'
 		# None if no info about peer
-		# False if peer does not support jep85
+		# False if peer does not support xep85
 		# 'ask' if we sent the first 'active' chatstate and are waiting for reply
 		# this holds what WE SEND to contact (our current chatstate)
 		self.our_chatstate = our_chatstate
 		self.msg_id = msg_id
-		# tell which JEP we're using for composing state
-		# None = have to ask, XEP-0022 = use this jep,
-		# XEP-0085 = use this jep, False = no composing support
+		# tell which XEP we're using for composing state
+		# None = have to ask, XEP-0022 = use this xep,
+		# XEP-0085 = use this xep, False = no composing support
 		self.composing_xep = composing_xep
 		# this is contact's chatstate
 		self.chatstate = chatstate
 		self.last_status_time = last_status_time
+		self.mood = mood.copy()
+		self.tune = tune.copy()
+		self.activity = activity.copy()
 
 	def get_full_jid(self):
 		if self.resource:
@@ -62,6 +87,18 @@ class Contact:
 			return self.contact_name
 		return self.jid.split('@')[0]
 
+	def get_shown_groups(self):
+		if self.is_observer():
+			return [_('Observers')]
+		elif self.is_groupchat():
+			return [_('Groupchats')]
+		elif self.is_transport():
+			return [_('Transports')]
+		elif not self.groups:
+			return [_('General')]
+		else:
+			return self.groups
+
 	def is_hidden_from_roster(self):
 		'''if contact should not be visible in roster'''
 		# XEP-0162: http://www.xmpp.org/extensions/xep-0162.html
@@ -85,6 +122,12 @@ class Contact:
 			is_observer = True
 		return is_observer
 
+	def is_groupchat(self):
+		for account in common.gajim.gc_connected:
+			if self.jid in common.gajim.gc_connected[account]:
+				return True
+		return False
+
 	def is_transport(self):
 		# if not '@' or '@' starts the jid then contact is transport
 		if self.jid.find('@') <= 0:
@@ -95,7 +138,8 @@ class Contact:
 class GC_Contact:
 	'''Information concerning each groupchat contact'''
 	def __init__(self, room_jid='', name='', show='', status='', role='',
-	affiliation='', jid = '', resource = ''):
+	affiliation='', jid = '', resource = '', our_chatstate = None,
+	composing_xep = None, chatstate = None):
 		self.room_jid = room_jid
 		self.name = name
 		self.show = show
@@ -104,6 +148,12 @@ class GC_Contact:
 		self.affiliation = affiliation
 		self.jid = jid
 		self.resource = resource
+		self.caps_node = None
+		self.caps_hash_method = None
+		self.caps_hash = None
+		self.our_chatstate = our_chatstate
+		self.composing_xep = composing_xep
+		self.chatstate = chatstate
 
 	def get_full_jid(self):
 		return self.room_jid + '/' + self.name
@@ -131,7 +181,7 @@ class Contacts:
 	def add_account(self, account):
 		self._contacts[account] = {}
 		self._gc_contacts[account] = {}
-		if not self._metacontacts_tags.has_key(account):
+		if account not in self._metacontacts_tags:
 			self._metacontacts_tags[account] = {}
 
 	def get_accounts(self):
@@ -143,27 +193,40 @@ class Contacts:
 		del self._metacontacts_tags[account]
 
 	def create_contact(self, jid='', name='', groups=[], show='', status='',
-		sub='', ask='', resource='', priority=0, keyID='', our_chatstate=None,
-		chatstate=None, last_status_time=None, composing_xep=None):
-		return Contact(jid, name, groups, show, status, sub, ask, resource,
-			priority, keyID, our_chatstate, chatstate, last_status_time,
-			composing_xep)
+		sub='', ask='', resource='', priority=0, keyID='', caps_node=None,
+		caps_hash_method=None, caps_hash=None, our_chatstate=None,
+		chatstate=None, last_status_time=None, composing_xep=None,
+		mood={}, tune={}, activity={}):
+	
+		# We don't want duplicated group values
+		groups_unique = []
+		for group in groups:
+			if group not in groups_unique:
+				groups_unique.append(group)
+
+		return Contact(jid=jid, name=name, groups=groups_unique, show=show,
+			status=status, sub=sub, ask=ask, resource=resource, priority=priority,
+			keyID=keyID, caps_node=caps_node, caps_hash_method=caps_hash_method,
+			caps_hash=caps_hash, our_chatstate=our_chatstate, chatstate=chatstate,
+			last_status_time=last_status_time, composing_xep=composing_xep,
+			mood=mood, tune=tune, activity=activity)
 	
 	def copy_contact(self, contact):
-		return self.create_contact(jid = contact.jid, name = contact.name,
-			groups = contact.groups, show = contact.show, status = contact.status,
-			sub = contact.sub, ask = contact.ask, resource = contact.resource,
-			priority = contact.priority, keyID = contact.keyID,
-			our_chatstate = contact.our_chatstate, chatstate = contact.chatstate,
-			last_status_time = contact.last_status_time)
+		return self.create_contact(jid=contact.jid, name=contact.name,
+			groups=contact.groups, show=contact.show, status=contact.status,
+			sub=contact.sub, ask=contact.ask, resource=contact.resource,
+			priority=contact.priority, keyID=contact.keyID,
+			caps_node=contact.caps_node, caps_hash_method=contact.caps_hash_method,
+			caps_hash=contact.caps_hash, our_chatstate=contact.our_chatstate,
+			chatstate=contact.chatstate, last_status_time=contact.last_status_time)
 
 	def add_contact(self, account, contact):
 		# No such account before ?
-		if not self._contacts.has_key(account):
+		if account not in self._contacts:
 			self._contacts[account] = {contact.jid : [contact]}
 			return
 		# No such jid before ?
-		if not self._contacts[account].has_key(contact.jid):
+		if contact.jid not in self._contacts[account]:
 			self._contacts[account][contact.jid] = [contact]
 			return
 		contacts = self._contacts[account][contact.jid]
@@ -180,9 +243,9 @@ class Contacts:
 		contacts.append(contact)
 
 	def remove_contact(self, account, contact):
-		if not self._contacts.has_key(account):
+		if account not in self._contacts:
 			return
-		if not self._contacts[account].has_key(contact.jid):
+		if contact.jid not in self._contacts[account]:
 			return
 		if contact in self._contacts[account][contact.jid]:
 			self._contacts[account][contact.jid].remove(contact)
@@ -192,38 +255,50 @@ class Contacts:
 	def clear_contacts(self, account):
 		self._contacts[account] = {}
 
-	def remove_jid(self, account, jid):
+	def remove_jid(self, account, jid, remove_meta=True):
 		'''Removes all contacts for a given jid'''
-		if not self._contacts.has_key(account):
+		if account not in self._contacts:
 			return
-		if not self._contacts[account].has_key(jid):
+		if jid not in self._contacts[account]:
 			return
 		del self._contacts[account][jid]
-		# remove metacontacts info
-		self.remove_metacontact(account, jid)
+		if remove_meta:
+			# remove metacontacts info
+			self.remove_metacontact(account, jid)
 
-	def get_contact(self, account, jid, resource = None):
-		'''Returns the list of contact instances for this jid (one per resource)
-		or [] if no resource is given
-		returns the contact instance for the given resource if it's given
-		or None if there is not'''
+	def get_contacts(self, account, jid):
+		'''Returns the list of contact instances for this jid.'''
+		if jid in self._contacts[account]:
+			return self._contacts[account][jid]
+		else:
+			return []
+	
+	def get_contact(self, account, jid, resource=None):
+		### WARNING ###
+		# This function returns a *RANDOM* resource if resource = None!
+		# Do *NOT* use if you need to get the contact to which you
+		# send a message for example, as a bare JID in Jabber means
+		# highest available resource, which this function ignores!
+		'''Returns the contact instance for the given resource if it's given else
+		the first contact is no resource is given or None if there is not'''
 		if jid in self._contacts[account]:
-			contacts = self._contacts[account][jid]
 			if not resource:
-				return contacts
-			for c in contacts:
+				return self._contacts[account][jid][0]
+			for c in self._contacts[account][jid]:
 				if c.resource == resource:
 					return c
-		if resource:
-			return None
-		return []
+		return None
 
-	def get_contacts_from_jid(self, account, jid):
-		'''we may have two or more resources on that jid'''
-		if jid in self._contacts[account]:
-			contacts_instances = self._contacts[account][jid]
-			return contacts_instances
-		return []
+	def iter_contacts(self, account):
+		if account in self._contacts:
+			for jid in self._contacts[account].keys():
+				for contact in self._contacts[account][jid][:]:
+					yield contact
+
+	def get_contact_from_full_jid(self, account, fjid):
+		''' Get Contact object for specific resource of given jid'''
+		barejid, resource = common.gajim.get_room_and_nick_from_fjid(fjid)
+		return self.get_contact(account, barejid, resource)
 
 	def get_highest_prio_contact_from_contacts(self, contacts):
 		if not contacts:
@@ -235,7 +310,7 @@ class Contacts:
 		return prim_contact
 
 	def get_contact_with_highest_priority(self, account, jid):
-		contacts = self.get_contacts_from_jid(account, jid)
+		contacts = self.get_contacts(account, jid)
 		if not contacts and '/' in jid:
 			# jid may be a fake jid, try it
 			room, nick = jid.split('/', 1)
@@ -252,12 +327,12 @@ class Contacts:
 		'''Returns all contacts in the given group'''
 		group_contacts = []
 		for jid in self._contacts[account]:
-			contacts = self.get_contacts_from_jid(account, jid)
+			contacts = self.get_contacts(account, jid)
 			if group in contacts[0].groups:
 				group_contacts += contacts
 		return group_contacts
 
-	def get_nb_online_total_contacts(self, accounts = [], groups = []):
+	def get_nb_online_total_contacts(self, accounts=[], groups=[]):
 		'''Returns the number of online contacts and the total number of
 		contacts'''
 		if accounts == []:
@@ -270,9 +345,13 @@ class Contacts:
 				if jid == our_jid:
 					continue
 				if common.gajim.jid_is_transport(jid) and not \
-					_('Transports') in groups:
+				_('Transports') in groups:
 					# do not count transports
 					continue
+				if self.has_brother(account, jid, accounts) and not \
+				self.is_big_brother(account, jid, accounts):
+					# count metacontacts only once
+					continue
 				contact = self.get_contact_with_highest_priority(account, jid)
 				if _('Not in roster') in contact.groups:
 					continue
@@ -280,16 +359,8 @@ class Contacts:
 				if groups == []:
 					in_groups = True
 				else:
-					contact_groups = contact.groups
-					if not contact_groups:
-						# Contact is not in a group, so count it in General or
-						# Transports group
-						if common.gajim.jid_is_transport(jid):
-							contact_groups = [_('Transports')]
-						else:
-							contact_groups = [_('General')]
 					for group in groups:
-						if group in contact_groups:
+						if group in contact.get_shown_groups():
 							in_groups = True
 							break
 
@@ -310,7 +381,7 @@ class Contacts:
 
 	def get_metacontacts_tag(self, account, jid):
 		'''Returns the tag of a jid'''
-		if not self._metacontacts_tags.has_key(account):
+		if not account in self._metacontacts_tags:
 			return None
 		for tag in self._metacontacts_tags[account]:
 			for data in self._metacontacts_tags[account][tag]:
@@ -332,7 +403,7 @@ class Contacts:
 		while old_tag:
 			self.remove_metacontact(account, jid)
 			old_tag = self.get_metacontacts_tag(account, jid)
-		if not self._metacontacts_tags[account].has_key(tag):
+		if tag not in self._metacontacts_tags[account]:
 			self._metacontacts_tags[account][tag] = [{'jid': jid, 'tag': tag}]
 		else:
 			self._metacontacts_tags[account][tag].append({'jid': jid,
@@ -348,23 +419,35 @@ class Contacts:
 					found = data
 					break
 			if found:
-				self._metacontacts_tags[account][tag].remove(data)
+				self._metacontacts_tags[account][tag].remove(found)
+				common.gajim.connections[account].store_metacontacts(
+					self._metacontacts_tags[account])
 				break
-		common.gajim.connections[account].store_metacontacts(
-			self._metacontacts_tags[account])
 
-	def has_brother(self, account, jid):
-		for account in self._metacontacts_tags:
-			tag = self.get_metacontacts_tag(account, jid)
-			if tag and len(self._metacontacts_tags[account][tag]) > 1:
+	def has_brother(self, account, jid, accounts):
+		tag = self.get_metacontacts_tag(account, jid)
+		if not tag:
+			return False
+		meta_jids = self.get_metacontacts_jids(tag, accounts)
+		return len(meta_jids) > 1 or len(meta_jids[account]) > 1
+
+	def is_big_brother(self, account, jid, accounts):
+		family = self.get_metacontacts_family(account, jid)
+		if family:
+			nearby_family = [data for data in family
+				if account in accounts]
+			bb_data = self.get_metacontacts_big_brother(nearby_family)
+			if bb_data['jid'] == jid and bb_data['account'] == account: 
 				return True
 		return False
 
-	def get_metacontacts_jids(self, tag):
+	def get_metacontacts_jids(self, tag, accounts):
 		'''Returns all jid for the given tag in the form {acct: [jid1, jid2],.}'''
 		answers = {}
 		for account in self._metacontacts_tags:
-			if self._metacontacts_tags[account].has_key(tag):
+			if tag in self._metacontacts_tags[account]:
+				if account not in accounts:
+					continue
 				answers[account] = []
 				for data in self._metacontacts_tags[account][tag]:
 					answers[account].append(data['jid'])
@@ -379,7 +462,7 @@ class Contacts:
 			return []
 		answers = []
 		for account in self._metacontacts_tags:
-			if self._metacontacts_tags[account].has_key(tag):
+			if tag in self._metacontacts_tags[account]:
 				for data in self._metacontacts_tags[account][tag]:
 					data['account'] = account
 					answers.append(data)
@@ -389,37 +472,76 @@ class Contacts:
 		'''compare 2 metacontacts.
 		Data is {'jid': jid, 'account': account, 'order': order}
 		order is optional'''
+		jid1 = data1['jid']
+		jid2 = data2['jid']
+		account1 = data1['account']
+		account2 = data2['account']
+		contact1 = self.get_contact_with_highest_priority(account1, jid1)
+		contact2 = self.get_contact_with_highest_priority(account2, jid2)
+		show_list = ['not in roster', 'error', 'offline', 'invisible', 'dnd',
+			'xa', 'away', 'chat', 'online', 'requested', 'message']
+		# contact can be null when a jid listed in the metacontact data
+		# is not in our roster
+		if not contact1:
+			if contact2:
+				return -1 # prefer the known contact 	
+			else:
+				show1 = 0
+				priority1 = 0
+		else:
+			show1 = show_list.index(contact1.show)
+			priority1 = contact1.priority
+		if not contact2:
+			if contact1:
+				return 1 # prefer the known contact
+			else:
+				show2 = 0
+				priority2 = 0
+		else:
+			show2 = show_list.index(contact2.show)
+			priority2 = contact2.priority
+		# If only one is offline, it's always second
+		if show1 > 2 and show2 < 3:
+			return 1
+		if show2 > 2 and show1 < 3:
+			return -1
 		if 'order' in data1 and 'order' in data2:
 			if data1['order'] > data2['order']:
 				return 1
 			if data1['order'] < data2['order']:
 				return -1
-		jid1 = data1['jid']
-		jid2 = data2['jid']
 		transport1 = common.gajim.get_transport_name_from_jid(jid1)
 		transport2 = common.gajim.get_transport_name_from_jid(jid2)
 		if transport2 and not transport1:
 			return 1
 		if transport1 and not transport2:
 			return -1
-		contact1 = self.get_contact_with_highest_priority(data1['account'], jid1)
-		contact2 = self.get_contact_with_highest_priority(data2['account'], jid2)
-		if contact1.priority > contact2.priority:
-			return 1
-		if contact2.priority > contact1.priority:
-			return -1
-		show_list = ['not in roster', 'error', 'offline', 'invisible', 'dnd',
-			'xa', 'away', 'chat', 'online', 'requested', 'message']
-		show1 = show_list.index(contact1.show)
-		show2 = show_list.index(contact2.show)
 		if show1 > show2:
 			return 1
 		if show2 > show1:
 			return -1
+		if priority1 > priority2:
+			return 1
+		if priority2 > priority1:
+			return -1
+		server1 = common.gajim.get_server_from_jid(jid1)
+		server2 = common.gajim.get_server_from_jid(jid2)
+		myserver1 = common.gajim.config.get_per('accounts', account1, 'hostname')
+		myserver2 = common.gajim.config.get_per('accounts', account2, 'hostname')
+		if server1 == myserver1:
+			if server2 != myserver2:
+				return 1
+		elif server2 == myserver2:
+			return -1
 		if jid1 > jid2:
 			return 1
 		if jid2 > jid1:
 			return -1
+		# If all is the same, compare accounts, they can't be the same
+		if account1 > account2:
+			return 1
+		if account2 > account1:
+			return -1
 		return 0
 
 	def get_metacontacts_big_brother(self, family):
@@ -446,9 +568,11 @@ class Contacts:
 	def contact_from_gc_contact(self, gc_contact):
 		'''Create a Contact instance from a GC_Contact instance'''
 		jid = gc_contact.get_full_jid()
-		return Contact(jid = jid, resource = '', name = gc_contact.name,
-			groups = [], show = gc_contact.show, status = gc_contact.status,
-			sub = 'none')
+		return Contact(jid=jid, resource='', name=gc_contact.name, groups=[],
+			show=gc_contact.show, status=gc_contact.status, sub='none',
+			caps_node=gc_contact.caps_node,
+			caps_hash_method=gc_contact.caps_hash_method,
+			caps_hash=gc_contact.caps_hash)
 
 	def create_gc_contact(self, room_jid='', name='', show='', status='',
 		role='', affiliation='', jid='', resource=''):
@@ -457,25 +581,25 @@ class Contacts:
 	
 	def add_gc_contact(self, account, gc_contact):
 		# No such account before ?
-		if not self._gc_contacts.has_key(account):
+		if account not in self._gc_contacts:
 			self._contacts[account] = {gc_contact.room_jid : {gc_contact.name: \
 				gc_contact}}
 			return
 		# No such room_jid before ?
-		if not self._gc_contacts[account].has_key(gc_contact.room_jid):
+		if gc_contact.room_jid not in self._gc_contacts[account]:
 			self._gc_contacts[account][gc_contact.room_jid] = {gc_contact.name: \
 				gc_contact}
 			return
 		self._gc_contacts[account][gc_contact.room_jid][gc_contact.name] = \
-				gc_contact
+			gc_contact
 
 	def remove_gc_contact(self, account, gc_contact):
-		if not self._gc_contacts.has_key(account):
+		if account not in self._gc_contacts:
 			return
-		if not self._gc_contacts[account].has_key(gc_contact.room_jid):
+		if gc_contact.room_jid not in self._gc_contacts[account]:
 			return
-		if not self._gc_contacts[account][gc_contact.room_jid].has_key(
-			gc_contact.name):
+		if gc_contact.name not in self._gc_contacts[account][
+		gc_contact.room_jid]:
 			return
 		del self._gc_contacts[account][gc_contact.room_jid][gc_contact.name]
 		# It was the last nick in room ?
@@ -483,14 +607,14 @@ class Contacts:
 			del self._gc_contacts[account][gc_contact.room_jid]
 
 	def remove_room(self, account, room_jid):
-		if not self._gc_contacts.has_key(account):
+		if account not in self._gc_contacts:
 			return
-		if not self._gc_contacts[account].has_key(room_jid):
+		if room_jid not in self._gc_contacts[account]:
 			return
 		del self._gc_contacts[account][room_jid]
 
 	def get_gc_list(self, account):
-		if not self._gc_contacts.has_key(account):
+		if account not in self._gc_contacts:
 			return []
 		return self._gc_contacts[account].keys()
 
@@ -505,3 +629,18 @@ class Contacts:
 		if not nick in nick_list:
 			return None
 		return self._gc_contacts[account][room_jid][nick]
+
+	def get_nb_role_total_gc_contacts(self, account, room_jid, role):
+		'''Returns the number of group chat contacts for the given role and the
+		total number of group chat contacts'''
+		if account not in self._gc_contacts:
+			return 0, 0
+		if room_jid not in self._gc_contacts[account]:
+			return 0, 0
+		nb_role = nb_total = 0
+		for nick in self._gc_contacts[account][room_jid]:
+			if self._gc_contacts[account][room_jid][nick].role == role:
+				nb_role += 1
+			nb_total += 1
+		return nb_role, nb_total
+# vim: se ts=3:
