@@ -351,7 +351,8 @@ class EncryptedStanzaSession(StanzaSession):
 		return self.decrypter.decrypt(ciphertext)
 
 	def logging_preference(self):
-		if gajim.config.get('log_encrypted_sessions'):
+		if gajim.config.get_per('accounts', self.conn.name,
+		'log_encrypted_sessions'):
 			return ['may', 'mustnot']
 		else:
 			return ['mustnot', 'may']
@@ -887,7 +888,10 @@ class EncryptedStanzaSession(StanzaSession):
 		srses = secrets.secrets().retained_secrets(self.conn.name,
 			self.jid.getStripped())
 
-		srshash = base64.b64decode(form['srshash'])
+		try:
+			srshash = base64.b64decode(form['srshash'])
+		except IndexError:
+			return
 
 		for (secret, verified) in srses:
 			if self.hmac(secret, 'Shared Retained Secret') == srshash:
