@@ -23,6 +23,15 @@ from auth import *
 from client import PlugIn
 import sha,base64,random,dispatcher_nb
 
+try:
+	import kerberos
+	have_kerberos = True
+except ImportError:
+	have_kerberos = False
+
+GSS_STATE_STEP = 0
+GSS_STATE_WRAP = 1
+
 def challenge_splitter(data):
 	''' Helper function that creates a dict from challenge string.
 	Sample chalenge string:
@@ -87,11 +96,11 @@ class SASL(PlugIn):
 	def __init__(self,username,password, on_sasl):
 		PlugIn.__init__(self)
 		self.username=username
-		self.password=password
+		self.password=password or ''
 		self.on_sasl = on_sasl
 		self.realm = None
 	def plugin(self,owner):
-		if not self._owner.Dispatcher.Stream._document_attrs.has_key('version'): 
+		if 'version' not in self._owner.Dispatcher.Stream._document_attrs: 
 			self.startsasl='not-supported'
 		elif self._owner.Dispatcher.Stream.features:
 			try: 
@@ -126,22 +135,46 @@ class SASL(PlugIn):
 			self.startsasl='not-supported'
 			self.DEBUG('SASL not supported by server', 'error')
 			return
-		mecs=[]
+		self.mecs=[]
 		for mec in feats.getTag('mechanisms', namespace=NS_SASL).getTags('mechanism'):
-			mecs.append(mec.getData())
+			self.mecs.append(mec.getData())
 		self._owner.RegisterHandler('challenge', self.SASLHandler, xmlns=NS_SASL)
 		self._owner.RegisterHandler('failure', self.SASLHandler, xmlns=NS_SASL)
 		self._owner.RegisterHandler('success', self.SASLHandler, xmlns=NS_SASL)
-		if "DIGEST-MD5" in mecs:
+		self.MechanismHandler()
+
+	def MechanismHandler(self):
+		if 'GSSAPI' in self.mecs and have_kerberos:
+			self.mecs.remove('GSSAPI')
+			try:
+				self.gss_vc = kerberos.authGSSClientInit('xmpp@' + \
+					self._owner.xmpp_hostname)[1]
+				kerberos.authGSSClientStep(self.gss_vc, '')
+				response = kerberos.authGSSClientResponse(self.gss_vc)
+				node=Node('auth',attrs={'xmlns': NS_SASL, 'mechanism': 'GSSAPI'},
+					payload=(response or ''))
+				self.mechanism = 'GSSAPI'
+				self.gss_step = GSS_STATE_STEP
+			except kerberos.GSSError, e:
+				self.DEBUG('GSSAPI authentication failed: %s' % str(e), 'warn')
+		elif 'DIGEST-MD5' in self.mecs:
+			self.mecs.remove('DIGEST-MD5')
 			node=Node('auth',attrs={'xmlns': NS_SASL, 'mechanism': 'DIGEST-MD5'})
-		elif "PLAIN" in mecs:
-			sasl_data='%s\x00%s\x00%s' % (self.username+'@' + self._owner.Server, 
-																	self.username, self.password)
-			node=Node('auth', attrs={'xmlns':NS_SASL,'mechanism':'PLAIN'}, 
-								payload=[base64.encodestring(sasl_data).replace('\n','')])
+			self.mechanism = 'DIGEST-MD5'
+		elif 'PLAIN' in self.mecs:
+			self.mecs.remove('PLAIN')
+			sasl_data = u'%s\x00%s\x00%s' % (self.username + '@' + \
+				self._owner.Server, self.username, self.password)
+			sasl_data = sasl_data.encode('utf-8').encode('base64').replace(
+				'\n','')
+			node = Node('auth', attrs={'xmlns': NS_SASL, 'mechanism': 'PLAIN'}, 
+				payload=[sasl_data])
+			self.mechanism = 'PLAIN'
 		else:
 			self.startsasl='failure'
-			self.DEBUG('I can only use DIGEST-MD5 and PLAIN mecanisms.', 'error')
+			self.DEBUG('I can only use DIGEST-MD5, GSSAPI and PLAIN mecanisms.', 'error')
+			if self.on_sasl:
+				self.on_sasl()
 			return
 		self.startsasl='in-process'
 		self._owner.send(node.__str__())
@@ -149,15 +182,19 @@ class SASL(PlugIn):
 
 	def SASLHandler(self, conn, challenge):
 		''' Perform next SASL auth step. Used internally. '''
-		if challenge.getNamespace() <> NS_SASL: 
+		if challenge.getNamespace() != NS_SASL: 
 			return
 		if challenge.getName() == 'failure':
 			self.startsasl = 'failure'
 			try: 
 				reason = challenge.getChildren()[0]
-			except: 
+			except Exception: 
 				reason = challenge
 			self.DEBUG('Failed SASL authentification: %s' % reason, 'error')
+			if len(self.mecs) > 0:
+				# There are other mechanisms to test
+				self.MechanismHandler()
+				raise NodeProcessed
 			if self.on_sasl :
 				self.on_sasl ()
 			raise NodeProcessed
@@ -176,11 +213,27 @@ class SASL(PlugIn):
 		incoming_data = challenge.getData()
 		data=base64.decodestring(incoming_data)
 		self.DEBUG('Got challenge:'+data,'ok')
+		if self.mechanism == 'GSSAPI':
+			if self.gss_step == GSS_STATE_STEP:
+				rc = kerberos.authGSSClientStep(self.gss_vc, incoming_data)
+				if rc != kerberos.AUTH_GSS_CONTINUE:
+					self.gss_step = GSS_STATE_WRAP
+			elif self.gss_step == GSS_STATE_WRAP:
+				rc = kerberos.authGSSClientUnwrap(self.gss_vc, incoming_data)
+				response = kerberos.authGSSClientResponse(self.gss_vc)
+				rc = kerberos.authGSSClientWrap(self.gss_vc, response,
+					kerberos.authGSSClientUserName(self.gss_vc))
+			response = kerberos.authGSSClientResponse(self.gss_vc)
+			if not response:
+				response = ''
+			self._owner.send(Node('response', attrs={'xmlns':NS_SASL},
+						payload=response).__str__())
+			raise NodeProcessed
 		chal = challenge_splitter(data)
-		if not self.realm and chal.has_key('realm'):
+		if not self.realm and 'realm' in chal:
 			self.realm = chal['realm']
-		if chal.has_key('qop') and ((type(chal['qop']) == str and \
-		chal['qop'] =='auth') or (type(chal['qop']) == list and 'auth' in \
+		if 'qop' in chal and ((isinstance(chal['qop'], str) and \
+		chal['qop'] =='auth') or (isinstance(chal['qop'], list) and 'auth' in \
 		chal['qop'])):
 			resp={}
 			resp['username'] = self.username
@@ -203,17 +256,17 @@ class SASL(PlugIn):
 						resp['qop'], HH(A2)]))
 			resp['response'] = response
 			resp['charset'] = 'utf-8'
-			sasl_data=''
-			for key in ['charset', 'username', 'realm', 'nonce', 'nc', 'cnonce', 'digest-uri', 'response', 'qop']:
+			sasl_data = u''
+			for key in ('charset', 'username', 'realm', 'nonce', 'nc', 'cnonce', 'digest-uri', 'response', 'qop'):
 				if key in ['nc','qop','response','charset']: 
-					sasl_data += "%s=%s," % (key,resp[key])
+					sasl_data += u"%s=%s," % (key,resp[key])
 				else: 
-					sasl_data += '%s="%s",' % (key,resp[key])
-########################################3333
-			node=Node('response', attrs={'xmlns':NS_SASL}, 
-				payload=[base64.encodestring(sasl_data[:-1]).replace('\r','').replace('\n','')])
+					sasl_data += u'%s="%s",' % (key,resp[key])
+			sasl_data = sasl_data[:-1].encode('utf-8').encode('base64').replace(
+				'\r','').replace('\n','')
+			node = Node('response', attrs={'xmlns':NS_SASL}, payload=[sasl_data])
 			self._owner.send(node.__str__())
-		elif chal.has_key('rspauth'): 
+		elif 'rspauth' in chal: 
 			self._owner.send(Node('response', attrs={'xmlns':NS_SASL}).__str__())
 		else: 
 			self.startsasl='failure'
@@ -231,7 +284,7 @@ class NonBlockingNonSASL(PlugIn):
 		PlugIn.__init__(self)
 		self.DBG_LINE ='gen_auth'
 		self.user = user
-		self.password= password
+		self.password= password or ''
 		self.resource = resource
 		self.on_auth = on_auth
 
@@ -347,14 +400,17 @@ class NonBlockingBind(Bind):
 				func=self._on_bound)
 	def _on_bound(self, resp):
 		if isResultNode(resp):
-			self.bound.append(resp.getTag('bind').getTagData('jid'))
-			self.DEBUG('Successfully bound %s.'%self.bound[-1],'ok')
-			jid=JID(resp.getTag('bind').getTagData('jid'))
-			self._owner.User=jid.getNode()
-			self._owner.Resource=jid.getResource()
-			self._owner.SendAndWaitForResponse(Protocol('iq', typ='set', 
-				payload=[Node('session', attrs={'xmlns':NS_SESSION})]), func=self._on_session)
-		elif resp:
+			if resp.getTag('bind') and resp.getTag('bind').getTagData('jid'):
+				self.bound.append(resp.getTag('bind').getTagData('jid'))
+				self.DEBUG('Successfully bound %s.'%self.bound[-1],'ok')
+				jid=JID(resp.getTag('bind').getTagData('jid'))
+				self._owner.User=jid.getNode()
+				self._owner.Resource=jid.getResource()
+				self._owner.SendAndWaitForResponse(Protocol('iq', typ='set', 
+					payload=[Node('session', attrs={'xmlns':NS_SESSION})]),
+					func=self._on_session)
+				return
+		if resp:
 			self.DEBUG('Binding failed: %s.' % resp.getTag('error'),'error')
 			self.on_bound(None)
 		else:
@@ -427,3 +483,5 @@ class NBComponentBind(ComponentBind):
 			self.DEBUG('Binding failed: timeout expired.', 'error')
 		if self.on_bind:
 			self.on_bind(None)
+
+# vim: se ts=3:
