@@ -97,8 +97,9 @@ class ZeroconfListener(IdleObject):
 				break;
 		P2PClient(sock[0], ipaddr, sock[1][1], self.conn_holder, [], from_jid)
 
-	def disconnect(self):
+	def disconnect(self, message=''):
 		''' free all resources, we are not listening anymore '''
+		log.info('Disconnecting ZeroconfListener: %s' % message)
 		gajim.idlequeue.remove_timeout(self.fd)
 		gajim.idlequeue.unplug_idle(self.fd)
 		self.fd = -1
@@ -698,8 +699,9 @@ class ClientZeroconf:
 		# look for hashed connections
 		if to in self.recipient_to_hash:
 			conn = self.connections[self.recipient_to_hash[to]]
-			id = conn.Dispatcher.getAnID()
-			stanza.setID(id)
+			if not stanza.getID():
+				id_ = conn.Dispatcher.getAnID()
+				stanza.setID(id_)
 			if conn.add_stanza(stanza, is_message):
 				if on_ok:
 					on_ok()
@@ -709,15 +711,17 @@ class ClientZeroconf:
 			hash = self.ip_to_hash[item['address']]
 			if self.hash_to_port[hash] == item['port']:
 				conn = self.connections[hash]
-				id = conn.Dispatcher.getAnID()
-				stanza.setID(id)
+				if not stanza.getID():
+					id_ = conn.Dispatcher.getAnID()
+					stanza.setID(id_)
 				if conn.add_stanza(stanza, is_message):
 					if on_ok:
 						on_ok()
 					return id
 
 		# otherwise open new connection
-		stanza.setID('zero')
+		if not stanza.getID():
+			stanza.setID('zero')
 		P2PClient(None, item['address'], item['port'], self,
 			[(stanza, is_message)], to, on_ok=on_ok, on_not_ok=on_not_ok)
 
