@@ -1,19 +1,27 @@
 # -*- coding: utf-8 -*-
-##	config.py
+## src/disco.py
 ##
-## Copyright (C) 2005-2006 Yann Le Boulanger <asterix@lagaule.org>
-## Copyright (C) 2005-2006 Nikos Kouremenos <kourem@gmail.com>
-## Copyright (C) 2005-2006 Stéphan Kochen <stephan@kochen.nl>
+## Copyright (C) 2005-2006 Stéphan Kochen <stephan AT kochen.nl>
+## Copyright (C) 2005-2007 Nikos Kouremenos <kourem AT gmail.com>
+## Copyright (C) 2005-2008 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2006 Dimitur Kirov <dkirov AT gmail.com>
+## Copyright (C) 2006-2008 Jean-Marie Traissard <jim AT lapin.org>
+## Copyright (C) 2007 Stephan Erb <steve-e AT h3c.de>
 ##
-## This program is free software; you can redistribute it and/or modify
+## This file is part of Gajim.
+##
+## Gajim is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 2 only.
+## by the Free Software Foundation; version 3 only.
 ##
-## This program is distributed in the hope that it will be useful,
+## Gajim is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ## GNU General Public License for more details.
 ##
+## You should have received a copy of the GNU General Public License
+## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+##
 
 # The appearance of the treeview, and parts of the dialog, are controlled by
 # AgentBrowser (sub-)classes. Methods that probably should be overridden when
@@ -37,7 +45,7 @@
 # There are more methods, of course, but this is a basic set.
 
 import os
-import inspect
+import types
 import weakref
 import gobject
 import gtk
@@ -48,10 +56,12 @@ import tooltips
 import gtkgui_helpers
 import groups
 import adhoc_commands
+import search_window
 
 from common import gajim
 from common import xmpp
 from common.exceptions import GajimGeneralException
+from common import helpers
 
 # Dictionary mapping category, type pairs to browser class, image pairs.
 # This is a function, so we can call it after the classes are declared.
@@ -65,6 +75,7 @@ def _gen_agent_type_info():
 		# Jabber server
 		('server', 'im'):				(ToplevelAgentBrowser, 'jabber.png'),
 		('services', 'jabber'):		(ToplevelAgentBrowser, 'jabber.png'),
+		('hierarchy', 'branch'):	(AgentBrowser, 'jabber.png'),
 
 		# Services
 		('conference', 'text'):		(MucBrowser, 'conference.png'),
@@ -74,9 +85,10 @@ def _gen_agent_type_info():
 		('_jid', 'weather'):			(False, 'weather.png'),
 		('gateway', 'sip'):			(False, 'sip.png'),
 		('directory', 'user'):		(None, 'jud.png'),
-		('pubsub', 'generic'):		(None, 'pubsub.png'),
+		('pubsub', 'generic'):		(PubSubBrowser, 'pubsub.png'),
 		('pubsub', 'service'):		(PubSubBrowser, 'pubsub.png'),
 		('proxy', 'bytestreams'):	(None, 'bytestreams.png'), # Socks5 FT proxy
+		('headline', 'newmail'):	(ToplevelAgentBrowser, 'mail.png'),
 
 		# Transports
 		('conference', 'irc'):		(ToplevelAgentBrowser, 'irc.png'),
@@ -95,6 +107,10 @@ def _gen_agent_type_info():
 		('gateway', 'smtp'):			(False, 'mail.png'),
 		('gateway', 'yahoo'):		(False, 'yahoo.png'),
 		('_jid', 'yahoo'):			(False, 'yahoo.png'),
+		('gateway', 'mrim'):			(False, 'mrim.png'),
+		('_jid', 'mrim'):				(False, 'mrim.png'),
+		('gateway', 'facebook'):	(False, 'facebook.png'),
+		('_jid', 'facebook'):		(False, 'facebook.png'),
 	}
 
 # Category type to "human-readable" description string, and sort priority
@@ -144,7 +160,7 @@ class CacheDictionary:
 		if item.source:
 			gobject.source_remove(item.source)
 		if self.lifetime:
-			source = gobject.timeout_add(self.lifetime, self._expire_timeout, key)
+			source = gobject.timeout_add_seconds(self.lifetime/1000, self._expire_timeout, key)
 			item.source = source
 
 	def __getitem__(self, key):
@@ -189,7 +205,7 @@ class Closure(object):
 		self.userargs = userargs
 		self.remove = remove
 		self.removeargs = removeargs
-		if inspect.ismethod(cb):
+		if isinstance(cb, types.MethodType):
 			self.meth_self = weakref.ref(cb.im_self, self._remove)
 			self.meth_name = cb.func_name
 		elif callable(cb):
@@ -226,9 +242,9 @@ class ServicesCache:
 		self._items.cleanup()
 		self._info.cleanup()
 
-	def _clean_closure(self, cb, type, addr):
+	def _clean_closure(self, cb, type_, addr):
 		# A closure died, clean up
-		cbkey = (type, addr)
+		cbkey = (type_, addr)
 		try:
 			self._cbs[cbkey].remove(cb)
 		except KeyError:
@@ -257,7 +273,7 @@ class ServicesCache:
 			info = _agent_type_info[(0, 0)]
 			filename = info[1]
 		if not filename: # we don't have an image to show for this type
-			return
+			filename = 'jabber.png'
 		# Use the cache if possible
 		if filename in _icon_cache:
 			return _icon_cache[filename]
@@ -268,36 +284,43 @@ class ServicesCache:
 		_icon_cache[filename] = pix
 		return pix
 
-	def get_browser(self, identities = [], features = []):
+	def get_browser(self, identities=[], features=[]):
 		'''Return the browser class for an agent.'''
-		# Grab the first identity with a browser
-		browser = None
+		# First pass, we try to find a ToplevelAgentBrowser
 		for identity in identities:
 			try:
-				cat, type = identity['category'], identity['type']
-				info = _agent_type_info[(cat, type)]
+				cat, type_ = identity['category'], identity['type']
+				info = _agent_type_info[(cat, type_)]
+			except KeyError:
+				continue
+			browser = info[0]
+			if browser and browser == ToplevelAgentBrowser:
+				return browser
+
+		# second pass, we haven't found a ToplevelAgentBrowser
+		for identity in identities:
+			try:
+				cat, type_ = identity['category'], identity['type']
+				info = _agent_type_info[(cat, type_)]
 			except KeyError:
 				continue
 			browser = info[0]
 			if browser:
-				break
-		# Note: possible outcome here is browser=False
-		if browser is None:
-			# NS_BROWSE is deprecated, but we check for it anyways.
-			# Some services list it in features and respond to
-			# NS_DISCO_ITEMS anyways.
-			# Allow browsing for unknown types aswell.
-			if (not features and not identities) or\
-					xmpp.NS_DISCO_ITEMS in features or\
-					xmpp.NS_BROWSE in features:
-				browser = AgentBrowser
-		return browser
+				return browser
+		# NS_BROWSE is deprecated, but we check for it anyways.
+		# Some services list it in features and respond to
+		# NS_DISCO_ITEMS anyways.
+		# Allow browsing for unknown types aswell.
+		if (not features and not identities) or \
+		xmpp.NS_DISCO_ITEMS in features or xmpp.NS_BROWSE in features:
+			return ToplevelAgentBrowser
+		return None
 
 	def get_info(self, jid, node, cb, force = False, nofetch = False, args = ()):
 		'''Get info for an agent.'''
 		addr = get_agent_address(jid, node)
 		# Check the cache
-		if self._info.has_key(addr):
+		if addr in self._info:
 			args = self._info[addr] + args
 			cb(jid, node, *args)
 			return
@@ -309,7 +332,7 @@ class ServicesCache:
 		cb = Closure(cb, userargs = args, remove = self._clean_closure,
 				removeargs = cbkey)
 		# Are we already fetching this?
-		if self._cbs.has_key(cbkey):
+		if cbkey in self._cbs:
 			self._cbs[cbkey].append(cb)
 		else:
 			self._cbs[cbkey] = [cb]
@@ -319,7 +342,7 @@ class ServicesCache:
 		'''Get a list of items in an agent.'''
 		addr = get_agent_address(jid, node)
 		# Check the cache
-		if self._items.has_key(addr):
+		if addr in self._items:
 			args = (self._items[addr],) + args
 			cb(jid, node, *args)
 			return
@@ -331,7 +354,7 @@ class ServicesCache:
 		cb = Closure(cb, userargs = args, remove = self._clean_closure,
 				removeargs = cbkey)
 		# Are we already fetching this?
-		if self._cbs.has_key(cbkey):
+		if cbkey in self._cbs:
 			self._cbs[cbkey].append(cb)
 		else:
 			self._cbs[cbkey] = [cb]
@@ -346,11 +369,11 @@ class ServicesCache:
 
 		# Call callbacks
 		cbkey = ('info', addr)
-		if self._cbs.has_key(cbkey):
+		if cbkey in self._cbs:
 			for cb in self._cbs[cbkey]:
 				cb(jid, node, identities, features, data)
 			# clean_closure may have beaten us to it
-			if self._cbs.has_key(cbkey):
+			if cbkey in self._cbs:
 				del self._cbs[cbkey]
 
 	def agent_items(self, jid, node, items):
@@ -362,11 +385,11 @@ class ServicesCache:
 
 		# Call callbacks
 		cbkey = ('items', addr)
-		if self._cbs.has_key(cbkey):
+		if cbkey in self._cbs:
 			for cb in self._cbs[cbkey]:
 				cb(jid, node, items)
 			# clean_closure may have beaten us to it
-			if self._cbs.has_key(cbkey):
+			if cbkey in self._cbs:
 				del self._cbs[cbkey]
 
 	def agent_info_error(self, jid):
@@ -376,11 +399,11 @@ class ServicesCache:
 
 		# Call callbacks
 		cbkey = ('info', addr)
-		if self._cbs.has_key(cbkey):
+		if cbkey in self._cbs:
 			for cb in self._cbs[cbkey]:
 				cb(jid, '', 0, 0, 0)
 			# clean_closure may have beaten us to it
-			if self._cbs.has_key(cbkey):
+			if cbkey in self._cbs:
 				del self._cbs[cbkey]
 
 	def agent_items_error(self, jid):
@@ -390,11 +413,11 @@ class ServicesCache:
 
 		# Call callbacks
 		cbkey = ('items', addr)
-		if self._cbs.has_key(cbkey):
+		if cbkey in self._cbs:
 			for cb in self._cbs[cbkey]:
 				cb(jid, '', 0)
 			# clean_closure may have beaten us to it
-			if self._cbs.has_key(cbkey):
+			if cbkey in self._cbs:
 				del self._cbs[cbkey]
 
 # object is needed so that @property works
@@ -436,17 +459,12 @@ _('Without a connection, you can not browse available services'))
 			self.on_services_treeview_selection_changed)
 		self.services_scrollwin = self.xml.get_widget('services_scrollwin')
 		self.progressbar = self.xml.get_widget('services_progressbar')
-		self.progressbar.set_no_show_all(True)
-		self.progressbar.hide()
 		self.banner = self.xml.get_widget('banner_agent_label')
 		self.banner_icon = self.xml.get_widget('banner_agent_icon')
 		self.banner_eventbox = self.xml.get_widget('banner_agent_eventbox')
 		self.style_event_id = 0
 		self.banner.realize()
 		self.paint_banner()
-		self.filter_hbox = self.xml.get_widget('filter_hbox')
-		self.filter_hbox.set_no_show_all(True)
-		self.filter_hbox.hide()
 		self.action_buttonbox = self.xml.get_widget('action_buttonbox')
 
 		# Address combobox
@@ -498,10 +516,7 @@ _('Without a connection, you can not browse available services'))
 		title_text = _('Service Discovery using account %s') % self.account
 		self.window.set_title(title_text)
 		self._set_window_banner_text(_('Service Discovery'))
-		if gtk.gtk_version >= (2, 8, 0) and gtk.pygtk_version >= (2, 8, 0):
-			self.banner_icon.clear()
-		else:
-			self.banner_icon.set_from_file(None)
+		self.banner_icon.clear()
 		self.banner_icon.hide() # Just clearing it doesn't work
 
 	def _set_window_banner_text(self, text, text_after = None):
@@ -593,7 +608,8 @@ _('Without a connection, you can not browse available services'))
 
 		# self.browser._get_agent_address() would break when no browser.
 		addr = get_agent_address(self.jid, self.node)
-		del gajim.interface.instances[self.account]['disco'][addr]
+		if addr in gajim.interface.instances[self.account]['disco']:
+			del gajim.interface.instances[self.account]['disco'][addr]
 
 		if self.browser:
 			self.window.hide()
@@ -623,7 +639,7 @@ _('Without a connection, you can not browse available services'))
 		# Update the window list
 		if self.jid:
 			old_addr = get_agent_address(self.jid, self.node)
-			if gajim.interface.instances[self.account]['disco'].has_key(old_addr):
+			if old_addr in gajim.interface.instances[self.account]['disco']:
 				del gajim.interface.instances[self.account]['disco'][old_addr]
 		addr = get_agent_address(jid, node)
 		gajim.interface.instances[self.account]['disco'][addr] = self
@@ -680,10 +696,22 @@ _('This type of service does not contain any items to browse.'))
 		if self.address_comboboxentry.get_active() != -1:
 			# user selected one of the entries so do auto-visit
 			jid = self.address_comboboxentry.child.get_text().decode('utf-8')
+			try:
+				jid = helpers.parse_jid(jid)
+			except helpers.InvalidFormat, s:
+				pritext = _('Invalid Server Name')
+				dialogs.ErrorDialog(pritext, str(s))
+				return
 			self.travel(jid, '')
 
 	def on_go_button_clicked(self, widget):
 		jid = self.address_comboboxentry.child.get_text().decode('utf-8')
+		try:
+			jid = helpers.parse_jid(jid)
+		except helpers.InvalidFormat, s:
+			pritext = _('Invalid Server Name')
+			dialogs.ErrorDialog(pritext, str(s))
+			return
 		if jid == self.jid: # jid has not changed
 			return
 		if jid in self.latest_addresses:
@@ -700,10 +728,12 @@ _('This type of service does not contain any items to browse.'))
 		self.travel(jid, '')
 
 	def on_services_treeview_row_activated(self, widget, path, col = 0):
-		self.browser.default_action()
+		if self.browser:
+			self.browser.default_action()
 
 	def on_services_treeview_selection_changed(self, widget):
-		self.browser.update_actions()
+		if self.browser:
+			self.browser.update_actions()
 
 
 class AgentBrowser:
@@ -726,8 +756,9 @@ class AgentBrowser:
 
 	def _set_initial_title(self):
 		'''Set the initial window title based on agent address.'''
-		self.window.window.set_title(_('Browsing %s using account %s') % \
-			(self._get_agent_address(), self.account))
+		self.window.window.set_title(_('Browsing %(address)s using account '
+			'%(account)s') % {'address': self._get_agent_address(),
+			'account': self.account})
 		self.window._set_window_banner_text(self._get_agent_address())
 
 	def _create_treemodel(self):
@@ -784,7 +815,7 @@ class AgentBrowser:
 	def _set_title(self, jid, node, identities, features, data):
 		'''Set the window title based on agent info.'''
 		# Set the banner and window title
-		if identities[0].has_key('name'):
+		if 'name' in identities[0]:
 			name = identities[0]['name']
 			self.window._set_window_banner_text(self._get_agent_address(), name)
 
@@ -934,6 +965,9 @@ _('This service does not contain any items to browse.'))
 		for item in items:
 			jid = item['jid']
 			node = item.get('node', '')
+			# If such an item is already here: don't add it
+			if self._find_item(jid, node):
+				continue
 			self._total_items += 1
 			self._add_item(jid, node, item, force)
 		self.window.services_treeview.set_model(self.model)
@@ -950,8 +984,7 @@ _('This service does not contain any items to browse.'))
 			self._update_error(iter, jid, node)
 		else:
 			# We got our info
-			self._update_info(iter, jid, node,
-				identities, features, data)
+			self._update_info(iter, jid, node, identities, features, data)
 		self.update_actions()
 
 	def _add_item(self, jid, node, item, force):
@@ -959,19 +992,22 @@ _('This service does not contain any items to browse.'))
 		disco#items query.'''
 		self.model.append((jid, node, item.get('name', ''),
 			get_agent_address(jid, node)))
+		self.cache.get_info(jid, node, self._agent_info, force = force)
 
-	def _update_item(self, iter, jid, node, item):
+	def _update_item(self, iter_, jid, node, item):
 		'''Called when an item should be updated in the model. The result of a
 		disco#items query. (seldom)'''
-		if item.has_key('name'):
-			self.model[iter][2] = item['name']
+		if 'name' in item:
+			self.model[iter_][2] = item['name']
 
-	def _update_info(self, iter, jid, node, identities, features, data):
+	def _update_info(self, iter_, jid, node, identities, features, data):
 		'''Called when an item should be updated in the model with further info.
 		The result of a disco#info query.'''
-		self.model[iter][2] = identities[0].get('name', '')
+		name = identities[0].get('name', '')
+		if name:
+			self.model[iter_][2] = name
 
-	def _update_error(self, iter, jid, node):
+	def _update_error(self, iter_, jid, node):
 		'''Called when a disco#info query failed for an item.'''
 		pass
 
@@ -988,25 +1024,26 @@ class ToplevelAgentBrowser(AgentBrowser):
 		self.register_button = None
 		self.join_button = None
 		self.execute_button = None
+		self.search_button = None
 		# Keep track of our treeview signals
 		self._view_signals = []
 		self._scroll_signal = None
 
-	def _pixbuf_renderer_data_func(self, col, cell, model, iter):
+	def _pixbuf_renderer_data_func(self, col, cell, model, iter_):
 		'''Callback for setting the pixbuf renderer's properties.'''
-		jid = model.get_value(iter, 0)
+		jid = model.get_value(iter_, 0)
 		if jid:
-			pix = model.get_value(iter, 2)
+			pix = model.get_value(iter_, 2)
 			cell.set_property('visible', True)
 			cell.set_property('pixbuf', pix)
 		else:
 			cell.set_property('visible', False)
 
-	def _text_renderer_data_func(self, col, cell, model, iter):
+	def _text_renderer_data_func(self, col, cell, model, iter_):
 		'''Callback for setting the text renderer's properties.'''
-		jid = model.get_value(iter, 0)
-		markup = model.get_value(iter, 3)
-		state = model.get_value(iter, 4)
+		jid = model.get_value(iter_, 0)
+		markup = model.get_value(iter_, 3)
+		state = model.get_value(iter_, 4)
 		cell.set_property('markup', markup)
 		if jid:
 			cell.set_property('cell_background_set', False)
@@ -1071,7 +1108,7 @@ class ToplevelAgentBrowser(AgentBrowser):
 			iter = None
 			try:
 				iter = self.model.get_iter(row)
-			except:
+			except Exception:
 				self.tooltip.hide_tooltip()
 				return
 			jid = self.model[iter][0]
@@ -1144,7 +1181,7 @@ class ToplevelAgentBrowser(AgentBrowser):
 		AgentBrowser._add_actions(self)
 		self.execute_button = gtk.Button()
 		image = gtk.image_new_from_stock(gtk.STOCK_EXECUTE, gtk.ICON_SIZE_BUTTON)
-		label = gtk.Label(_('_Execute Command...'))
+		label = gtk.Label(_('_Execute Command'))
 		label.set_use_underline(True)
 		hbox = gtk.HBox()
 		hbox.pack_start(image, False, True, 6)
@@ -1172,6 +1209,18 @@ class ToplevelAgentBrowser(AgentBrowser):
 		self.window.action_buttonbox.add(self.join_button)
 		self.join_button.show_all()
 
+		self.search_button = gtk.Button()
+		image = gtk.image_new_from_stock(gtk.STOCK_FIND, gtk.ICON_SIZE_BUTTON)
+		label = gtk.Label(_('_Search'))
+		label.set_use_underline(True)
+		hbox = gtk.HBox()
+		hbox.pack_start(image, False, True, 6)
+		hbox.pack_end(label, True, True)
+		self.search_button.add(hbox)
+		self.search_button.connect('clicked', self.on_search_button_clicked)
+		self.window.action_buttonbox.add(self.search_button)
+		self.search_button.show_all()
+
 	def _clean_actions(self):
 		if self.execute_button:
 			self.execute_button.destroy()
@@ -1182,8 +1231,25 @@ class ToplevelAgentBrowser(AgentBrowser):
 		if self.join_button:
 			self.join_button.destroy()
 			self.join_button = None
+		if self.search_button:
+			self.search_button.destroy()
+			self.search_button = None
 		AgentBrowser._clean_actions(self)
 
+	def on_search_button_clicked(self, widget = None):
+		'''When we want to search something:
+		open search window'''
+		model, iter = self.window.services_treeview.get_selection().get_selected()
+		if not iter:
+			return
+		service = model[iter][0].decode('utf-8')
+		if service in gajim.interface.instances[self.account]['search']:
+			gajim.interface.instances[self.account]['search'][service].window.\
+				present()
+		else:
+			gajim.interface.instances[self.account]['search'][service] = \
+				search_window.SearchWindow(self.account, service)
+
 	def cleanup(self):
 		self.tooltip.hide_tooltip()
 		AgentBrowser.cleanup(self)
@@ -1195,14 +1261,15 @@ class ToplevelAgentBrowser(AgentBrowser):
 			self._renderer.set_property('cell-background', bgcolor)
 		self.window.services_treeview.queue_draw()
 
-	def on_execute_button_clicked(self, widget = None):
+	def on_execute_button_clicked(self, widget=None):
 		'''When we want to execute a command:
 		open adhoc command window'''
 		model, iter = self.window.services_treeview.get_selection().get_selected()
 		if not iter:
 			return
 		service = model[iter][0].decode('utf-8')
-		adhoc_commands.CommandWindow(self.account, service)
+		node = model[iter][1].decode('utf-8')
+		adhoc_commands.CommandWindow(self.account, service, commandnode=node)
 
 	def on_register_button_clicked(self, widget = None):
 		'''When we want to register an agent:
@@ -1223,7 +1290,7 @@ class ToplevelAgentBrowser(AgentBrowser):
 		if not iter:
 			return
 		service = model[iter][0].decode('utf-8')
-		if not gajim.interface.instances[self.account].has_key('join_gc'):
+		if 'join_gc' not in gajim.interface.instances[self.account]:
 			try:
 				dialogs.JoinGroupchatWindow(self.account, service)
 			except GajimGeneralException:
@@ -1241,6 +1308,9 @@ class ToplevelAgentBrowser(AgentBrowser):
 			self.browse_button.set_sensitive(False)
 		if self.join_button:
 			self.join_button.set_sensitive(False)
+		if self.search_button:
+			self.search_button.set_sensitive(False)
+		model, iter = self.window.services_treeview.get_selection().get_selected()
 		model, iter = self.window.services_treeview.get_selection().get_selected()
 		if not iter:
 			return
@@ -1273,6 +1343,8 @@ class ToplevelAgentBrowser(AgentBrowser):
 		AgentBrowser._update_actions(self, jid, node, identities, features, data)
 		if self.execute_button and xmpp.NS_COMMANDS in features:
 			self.execute_button.set_sensitive(True)
+		if self.search_button and xmpp.NS_SEARCH in features:
+			self.search_button.set_sensitive(True)
 		if self.register_button and xmpp.NS_REGISTER in features:
 			# We can register this agent
 			registered_transports = []
@@ -1322,17 +1394,17 @@ class ToplevelAgentBrowser(AgentBrowser):
 
 		fraction = 0
 		if self._total_items:
-			self.window.progressbar.set_text(_("Scanning %d / %d..") %\
-				(self._progress, self._total_items))
+			self.window.progressbar.set_text(_("Scanning %(current)d / %(total)d.."
+				) % {'current': self._progress, 'total': self._total_items})
 			fraction = float(self._progress) / float(self._total_items)
 			if self._progress >= self._total_items:
 				# We show the progressbar for just a bit before hiding it.
-				id = gobject.timeout_add(1500, self._hide_progressbar_cb)
+				id = gobject.timeout_add_seconds(2, self._hide_progressbar_cb)
 				self._progressbar_sourceid = id
 			else:
 				self.window.progressbar.show()
 				# Hide the progressbar if we're timing out anyways. (20 secs)
-				id = gobject.timeout_add(20000, self._hide_progressbar_cb)
+				id = gobject.timeout_add_seconds(20, self._hide_progressbar_cb)
 				self._progressbar_sourceid = id
 		self.window.progressbar.set_fraction(fraction)
 
@@ -1342,13 +1414,13 @@ class ToplevelAgentBrowser(AgentBrowser):
 			self.window.progressbar.hide()
 		return False
 
-	def _friendly_category(self, category, type=None):
+	def _friendly_category(self, category, type_=None):
 		'''Get the friendly category name and priority.'''
 		cat = None
-		if type:
+		if type_:
 			# Try type-specific override
 			try:
-				cat, prio = _cat_to_descr[(category, type)]
+				cat, prio = _cat_to_descr[(category, type_)]
 			except KeyError:
 				pass
 		if not cat:
@@ -1358,14 +1430,14 @@ class ToplevelAgentBrowser(AgentBrowser):
 				cat, prio = _cat_to_descr['other']
 		return cat, prio
 
-	def _create_category(self, cat, type=None):
+	def _create_category(self, cat, type_=None):
 		'''Creates a category row.'''
-		cat, prio = self._friendly_category(cat, type)
+		cat, prio = self._friendly_category(cat, type_)
 		return self.model.append(None, ('', '', None, cat, prio))
 
-	def _find_category(self, cat, type=None):
+	def _find_category(self, cat, type_=None):
 		'''Looks up a category row and returns the iterator to it, or None.'''
-		cat, prio = self._friendly_category(cat, type)
+		cat, prio = self._friendly_category(cat, type_)
 		iter = self.model.get_iter_root()
 		while iter:
 			if self.model.get_value(iter, 3).decode('utf-8') == cat:
@@ -1394,7 +1466,7 @@ class ToplevelAgentBrowser(AgentBrowser):
 	def _add_item(self, jid, node, item, force):
 		# Row text
 		addr = get_agent_address(jid, node)
-		if item.has_key('name'):
+		if 'name' in item:
 			descr = "<b>%s</b>\n%s" % (item['name'], addr)
 		else:
 			descr = "<b>%s</b>" % addr
@@ -1421,15 +1493,15 @@ class ToplevelAgentBrowser(AgentBrowser):
 		self.cache.get_info(jid, node, self._agent_info, force = force)
 		self._update_progressbar()
 
-	def _update_item(self, iter, jid, node, item):
+	def _update_item(self, iter_, jid, node, item):
 		addr = get_agent_address(jid, node)
-		if item.has_key('name'):
+		if 'name' in item:
 			descr = "<b>%s</b>\n%s" % (item['name'], addr)
 		else:
 			descr = "<b>%s</b>" % addr
-		self.model[iter][3] = descr
+		self.model[iter_][3] = descr
 
-	def _update_info(self, iter, jid, node, identities, features, data):
+	def _update_info(self, iter_, jid, node, identities, features, data):
 		addr = get_agent_address(jid, node)
 		name = identities[0].get('name', '')
 		if name:
@@ -1451,16 +1523,16 @@ class ToplevelAgentBrowser(AgentBrowser):
 			break
 
 		# Check if we have to move categories
-		old_cat_iter = self.model.iter_parent(iter)
+		old_cat_iter = self.model.iter_parent(iter_)
 		old_cat = self.model.get_value(old_cat_iter, 3).decode('utf-8')
 		if self.model.get_value(old_cat_iter, 3) == cat:
 			# Already in the right category, just update
-			self.model[iter][2] = pix
-			self.model[iter][3] = descr
-			self.model[iter][4] = 0
+			self.model[iter_][2] = pix
+			self.model[iter_][3] = descr
+			self.model[iter_][4] = 0
 			return
 		# Not in the right category, move it.
-		self.model.remove(iter)
+		self.model.remove(iter_)
 
 		# Check if the old category is empty
 		if not self.model.iter_is_valid(old_cat_iter):
@@ -1474,9 +1546,9 @@ class ToplevelAgentBrowser(AgentBrowser):
 		self.model.append(cat_iter, (jid, node, pix, descr, 0))
 		self._expand_all()
 
-	def _update_error(self, iter, jid, node):
+	def _update_error(self, iter_, jid, node):
 		addr = get_agent_address(jid, node)
-		self.model[iter][4] = 2
+		self.model[iter_][4] = 2
 		self._progress += 1
 		self._update_progressbar()
 
@@ -1622,7 +1694,8 @@ class MucBrowser(AgentBrowser):
 		# Bottom row
 		# Iter compare is broke, use the path instead
 		try:
-			ex, ey = view.tree_to_widget_coords(rect.x + rect.height, rect.y + rect.height)
+			ex, ey = view.tree_to_widget_coords(rect.x + rect.height,
+				rect.y + rect.height)
 			end = view.get_path_at_pos(ex, ey)[0]
 			# end is the last visible, we want to query that aswell
 			end = (end[0] + 1,)
@@ -1672,25 +1745,25 @@ class MucBrowser(AgentBrowser):
 		if not self._fetch_source:
 			self._fetch_source = gobject.idle_add(self._start_info_query)
 
-	def _update_info(self, iter, jid, node, identities, features, data):
+	def _update_info(self, iter_, jid, node, identities, features, data):
 		name = identities[0].get('name', '')
 		for form in data:
 			typefield = form.getField('FORM_TYPE')
-			if typefield and typefield.getValue() ==\
-					'http://jabber.org/protocol/muc#roominfo':
+			if typefield and typefield.getValue() == \
+			'http://jabber.org/protocol/muc#roominfo':
 				# Fill model row from the form's fields
 				users = form.getField('muc#roominfo_occupants')
 				descr = form.getField('muc#roominfo_description')
 				if users:
-					self.model[iter][3] = int(users.getValue())
-					self.model[iter][4] = users.getValue()
+					self.model[iter_][3] = int(users.getValue())
+					self.model[iter_][4] = users.getValue()
 				if descr:
-					self.model[iter][5] = descr.getValue()
+					self.model[iter_][5] = descr.getValue()
 				# Only set these when we find a form with additional info
 				# Some servers don't support forms and put extra info in
 				# the name attribute, so we preserve it in that case.
-				self.model[iter][2] = name
-				self.model[iter][6] = True
+				self.model[iter_][2] = name
+				self.model[iter_][6] = True
 				break
 		else:
 			# We didn't find a form, switch to alternate query mode
@@ -1700,7 +1773,7 @@ class MucBrowser(AgentBrowser):
 		self._fetch_source = None
 		self._query_visible()
 
-	def _update_error(self, iter, jid, node):
+	def _update_error(self, iter_, jid, node):
 		# switch to alternate query mode
 		self.cache.get_items(jid, node, self._channel_altinfo)
 
@@ -1731,8 +1804,9 @@ class DiscussionGroupsBrowser(AgentBrowser):
 	def _create_treemodel(self):
 		''' Create treemodel for the window. '''
 		# JID, node, name (with description) - pango markup, dont have info?, subscribed?
-		self.model = gtk.ListStore(str, str, str, bool, bool)
-		self.model.set_sort_column_id(3, gtk.SORT_ASCENDING)
+		self.model = gtk.TreeStore(str, str, str, bool, bool)
+		# sort by name
+		self.model.set_sort_column_id(2, gtk.SORT_ASCENDING)
 		self.window.services_treeview.set_model(self.model)
 
 		# Name column
@@ -1754,7 +1828,29 @@ class DiscussionGroupsBrowser(AgentBrowser):
 		col.set_resizable(False)
 		self.window.services_treeview.insert_column(col, -1)
 
-		self.window.services_treeview.set_headers_visible(True)
+		# Node Column
+		renderer = gtk.CellRendererText()
+		col = gtk.TreeViewColumn(_('Node'))
+		col.pack_start(renderer)
+		col.set_attributes(renderer, markup=1)
+		col.set_resizable(True)
+		self.window.services_treeview.insert_column(col, -1)
+
+	def _add_items(self, jid, node, items, force):
+		for item in items:
+			jid = item['jid']
+			node = item.get('node', '')
+			self._total_items += 1
+			self._add_item(jid, node, item, force)
+
+	def _in_list_foreach(self, model, path, iter_, node):
+		if model[path][1] == node:
+			self.in_list = True
+
+	def _in_list(self, node):
+		self.in_list = False
+		self.model.foreach(self._in_list_foreach, node)
+		return self.in_list
 
 	def _add_item(self, jid, node, item, force):
 		''' Called when we got basic information about new node from query.
@@ -1768,10 +1864,27 @@ class DiscussionGroupsBrowser(AgentBrowser):
 			dunno = True
 			subscribed = False
 
-		name = gtkgui_helpers.escape_for_pango_markup(name)
+		name = gobject.markup_escape_text(name)
 		name = '<b>%s</b>' % name
 
-		self.model.append((jid, node, name, dunno, subscribed))
+		node_splitted = node.split('/')
+		parent_iter = None
+		while len(node_splitted) > 1:
+			parent_node = node_splitted.pop(0)
+			parent_iter = self._get_child_iter(parent_iter, parent_node)
+			node_splitted[0] = parent_node + '/' + node_splitted[0]
+		if not self._in_list(node):
+			self.model.append(parent_iter, (jid, node, name, dunno, subscribed))
+			self.cache.get_items(jid, node, self._add_items, force = force,
+				args = (force,))
+
+	def _get_child_iter(self, parent_iter, node):
+		child_iter = self.model.iter_children(parent_iter)
+		while child_iter:
+			if self.model[child_iter][1] == node:
+				return child_iter
+			child_iter = self.model.iter_next(child_iter)
+		return None
 
 	def _add_actions(self):
 		self.post_button = gtk.Button(label=_('New post'), use_underline=True)
@@ -1855,7 +1968,7 @@ class DiscussionGroupsBrowser(AgentBrowser):
 		have items on the list, we should actualize them. '''
 		try:
 			subscriptions = request.getTag('pubsub').getTag('subscriptions')
-		except:
+		except Exception:
 			return 
 
 		groups = set()
@@ -1909,3 +2022,5 @@ class DiscussionGroupsBrowser(AgentBrowser):
 
 # Fill the global agent type info dictionary
 _agent_type_info = _gen_agent_type_info()
+
+# vim: se ts=3:
