@@ -228,8 +228,7 @@ _('Connection with peer cannot be established.'))
 		sectext += '\n\t' + _('Recipient: %s') % jid
 		if error_msg:
 			sectext += '\n\t' + _('Error message: %s') % error_msg
-		dialogs.ErrorDialog(_('File transfer stopped by the contact at the other '
-			'end'), sectext)
+		dialogs.ErrorDialog(_('File transfer stopped'), sectext)
 		self.tree.get_selection().unselect_all()
 
 	def show_file_send_request(self, account, contact):
@@ -308,7 +307,8 @@ _('Connection with peer cannot be established.'))
 		file requested by a contact'''
 		if file_props is None or 'name' not in file_props:
 			return
-		sec_text = '\t' + _('File: %s') % file_props['name']
+		sec_text = '\t' + _('File: %s') % gobject.markup_escape_text(
+			file_props['name'])
 		if 'size' in file_props:
 			sec_text += '\n\t' + _('Size: %s') % \
 				helpers.convert_bytes(file_props['size'])
