@@ -1059,8 +1059,8 @@ class GroupchatControl(ChatControlBase):
 				color = gtkgui_helpers._get_fade_color(self.list_treeview,
 					selected, focus)
 				colorstring = "#%04x%04x%04x" % (color.red, color.green, color.blue)
-				name += '\n' '<span size="small" style="italic" foreground="%s">%s</span>'\
-					% (colorstring, gobject.markup_escape_text(status))
+				name += ('\n<span size="small" style="italic" foreground="%s">'
+					'%s</span>') % (colorstring, gobject.markup_escape_text(status))
 
 		model[iter][C_IMG] = image
 		model[iter][C_TEXT] = name
@@ -1159,8 +1159,9 @@ class GroupchatControl(ChatControlBase):
 							'reason': reason }
 					self.print_conversation(s, 'info', tim = tim)
 				elif '303' in statusCode: # Someone changed his or her nick
-					if new_nick == self.new_nick: # We changed our nick
-						self.nick = self.new_nick
+					if new_nick == self.new_nick or nick == self.nick:
+						# We changed our nick
+						self.nick = new_nick
 						self.new_nick = ''
 						s = _('You are now known as %s') % new_nick
 					else:
@@ -1236,6 +1237,11 @@ class GroupchatControl(ChatControlBase):
 		else:
 			iter = self.get_contact_iter(nick)
 			if not iter:
+				if '210' in statusCode:
+					# Server changed our nick
+					self.nick = nick
+					s = _('You are now known as %s') % nick
+					self.print_conversation(s, 'info', tim=tim)
 				iter = self.add_contact_to_roster(nick, show, role, affiliation,
 					status, jid)
 				newly_created = True
@@ -1455,7 +1461,8 @@ class GroupchatControl(ChatControlBase):
 					dialogs.ErrorDialog(_('Invalid nickname'),
 					_('The nickname has not allowed characters.'))
 					return True
-				gajim.connections[self.account].join_gc(nick, self.room_jid, None)
+				gajim.connections[self.account].join_gc(nick, self.room_jid, None,
+					change_nick=True)
 				self.new_nick = nick
 				self.clear(self.msg_textview)
 			else:
@@ -1736,7 +1743,8 @@ class GroupchatControl(ChatControlBase):
 				dialogs.ErrorDialog(_('Invalid nickname'),
 				_('The nickname has not allowed characters.'))
 				return
-			gajim.connections[self.account].join_gc(nick, self.room_jid, None)
+			gajim.connections[self.account].join_gc(nick, self.room_jid, None,
+				change_nick=True)
 			if gajim.gc_connected[self.account][self.room_jid]:
 				# We are changing nick, we will change self.nick when we receive
 				# presence that inform that it works
@@ -1774,7 +1782,8 @@ class GroupchatControl(ChatControlBase):
 	def shutdown(self, status='offline'):
 		# destroy banner tooltip - bug #pygtk for that!
 		self.subject_tooltip.destroy()
-		if gajim.gc_connected[self.account][self.room_jid]:
+		if self.room_jid in gajim.gc_connected[self.account] and \
+		gajim.gc_connected[self.account][self.room_jid]:
 			# Tell connection to note the date we disconnect to avoid duplicate 
 			# logs. We do it only when connected because if connection was lost
 			# there may be new messages since disconnection.
