@@ -1,19 +1,24 @@
-##	gtkexcepthook.py
+# -*- coding:utf-8 -*-
+## src/gtkexcepthook.py
 ##
-## Copyright (C) 2005-2006 Yann Le Boulanger <asterix@lagaule.org>
-## Copyright (C) 2005-2006 Nikos Kouremenos <kourem@gmail.com>
+## Copyright (C) 2005-2006 Nikos Kouremenos <kourem AT gmail.com>
+## Copyright (C) 2005-2008 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2008 Stephan Erb <steve-e AT h3c.de>
 ##
-## Initially written and submitted by Gustavo J. A. M. Carneiro
+## This file is part of Gajim.
 ##
-## This program is free software; you can redistribute it and/or modify
+## Gajim is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 2 only.
+## by the Free Software Foundation; version 3 only.
 ##
-## This program is distributed in the hope that it will be useful,
+## Gajim is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ## GNU General Public License for more details.
 ##
+## You should have received a copy of the GNU General Public License
+## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+##
 
 import sys
 import os
@@ -30,11 +35,11 @@ from common import helpers
 
 _exception_in_progress = threading.Lock()
 
-def _info(type, value, tb):
+def _info(type_, value, tb):
 	if not _exception_in_progress.acquire(False):
 		# Exceptions have piled up, so we use the default exception
 		# handler for such exceptions
-		_excepthook_save(type, value, tb)
+		_excepthook_save(type_, value, tb)
 		return
 
 	dialog = dialogs.HigDialog(None, gtk.MESSAGE_WARNING, gtk.BUTTONS_NONE, 
@@ -42,6 +47,7 @@ def _info(type, value, tb):
 				_('It probably is not fatal, but should be reported '
 				'to the developers nonetheless.'))
 	
+	dialog.set_modal(False)
 	#FIXME: add icon to this button
 	RESPONSE_REPORT_BUG = 42
 	dialog.add_buttons(gtk.STOCK_CLOSE, gtk.BUTTONS_CLOSE,
@@ -63,7 +69,7 @@ def _info(type, value, tb):
 	frame.set_border_width(6)
 	textbuffer = textview.get_buffer()
 	trace = StringIO()
-	traceback.print_exception(type, value, tb, None, trace)
+	traceback.print_exception(type_, value, tb, None, trace)
 	textbuffer.set_text(trace.getvalue())
 	textview.set_size_request(
 		gtk.gdk.screen_width() / 3,
@@ -76,18 +82,15 @@ def _info(type, value, tb):
 	# on expand the details the dialog remains centered on screen
 	dialog.set_position(gtk.WIN_POS_CENTER_ALWAYS)
 
-	dialog.show_all()
-
 	close_clicked = False
-	while not close_clicked:
-		resp = dialog.run()
-		if resp == RESPONSE_REPORT_BUG:
-			url = 'http://trac.gajim.org/wiki/WikiStart#howto_report_ticket'
+	def on_dialog_response(dialog, response):
+		if response == RESPONSE_REPORT_BUG:
+			url = 'http://trac.gajim.org/wiki/HowToCreateATicket'
 			helpers.launch_browser_mailer('url', url)
 		else:
-			close_clicked = True
-	
-	dialog.destroy()
+			dialog.destroy()
+	dialog.connect('response', on_dialog_response)
+	dialog.show_all()
 
 	_exception_in_progress.release()
 	
@@ -102,3 +105,5 @@ if __name__ == '__main__':
 	_excepthook_save = sys.excepthook
 	sys.excepthook = _info
 	print x # this always tracebacks
+
+# vim: se ts=3:
