@@ -1,17 +1,27 @@
-#!/usr/bin/env python
-## history_manager.py
+# -*- coding:utf-8 -*-
+## src/history_manager.py
 ##
-## Copyright (C) 2006 Nikos Kouremenos <kourem@gmail.com>
+## Copyright (C) 2006 Dimitur Kirov <dkirov AT gmail.com>
+## Copyright (C) 2006-2007 Jean-Marie Traissard <jim AT lapin.org>
+##                         Nikos Kouremenos <kourem AT gmail.com>
+## Copyright (C) 2006-2008 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2007 Stephan Erb <steve-e AT h3c.de>
+## Copyright (C) 2008 Jonathan Schleifer <js-gajim AT webkeks.org>
 ##
-## This program is free software; you can redistribute it and/or modify
+## This file is part of Gajim.
+##
+## Gajim is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 2 only.
+## by the Free Software Foundation; version 3 only.
 ##
-## This program is distributed in the hope that it will be useful,
+## Gajim is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ## GNU General Public License for more details.
 ##
+## You should have received a copy of the GNU General Public License
+## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+##
 
 ## NOTE: some method names may match those of logger.py but that's it
 ## someday (TM) should have common class that abstracts db connections and helpers on it
@@ -23,26 +33,56 @@ if os.name == 'nt':
 	import warnings
 	warnings.filterwarnings(action='ignore')
 
-# Used to create windows installer with GTK included
-#	paths = os.environ['PATH']
-#	list_ = paths.split(';')
-#	new_list = []
-#	for p in list_:
-#		if p.find('gtk') < 0 and p.find('GTK') < 0:
-#			new_list.append(p)
-#	new_list.insert(0, 'gtk/lib')
-#	new_list.insert(0, 'gtk/bin')
-#	os.environ['PATH'] = ';'.join(new_list)
-#	os.environ['GTK_BASEPATH'] = 'gtk'
+	if os.path.isdir('gtk'):
+		# Used to create windows installer with GTK included
+		paths = os.environ['PATH']
+		list_ = paths.split(';')
+		new_list = []
+		for p in list_:
+			if p.find('gtk') < 0 and p.find('GTK') < 0:
+				new_list.append(p)
+		new_list.insert(0, 'gtk/lib')
+		new_list.insert(0, 'gtk/bin')
+		os.environ['PATH'] = ';'.join(new_list)
+		os.environ['GTK_BASEPATH'] = 'gtk'
 
 import sys
 import signal
 import gtk
+import gobject
 import time
 import locale
 
+import getopt
 from common import i18n
-import exceptions
+
+def parseOpts():
+	config_path = None
+
+	try:
+		shortargs = 'hc:'
+		longargs = 'help config_path='
+		opts = getopt.getopt(sys.argv[1:], shortargs, longargs.split())[0]
+	except getopt.error, msg:
+		print str(msg)
+		print 'for help use --help'
+		sys.exit(2)
+	for o, a in opts:
+		if o in ('-h', '--help'):
+			print 'history_manager [--help] [--config-path]'
+			sys.exit()
+		elif o in ('-c', '--config-path'):
+			config_path = a
+	return config_path
+
+config_path = parseOpts()
+del parseOpts
+
+import common.configpaths 
+common.configpaths.gajimpaths.init(config_path)
+del config_path
+common.configpaths.gajimpaths.init_profile()
+from common import exceptions
 import dialogs
 import gtkgui_helpers
 from common.logger import LOG_DB_PATH, constants
@@ -91,11 +131,8 @@ class HistoryManager:
 		self.logs_scrolledwindow = xml.get_widget('logs_scrolledwindow')
 		self.search_results_scrolledwindow = xml.get_widget(
 			'search_results_scrolledwindow')
-		self.welcome_label = xml.get_widget('welcome_label')
-			
-		self.logs_scrolledwindow.set_no_show_all(True)
-		self.search_results_scrolledwindow.set_no_show_all(True)
-		
+		self.welcome_vbox = xml.get_widget('welcome_vbox')
+
 		self.jids_already_in = [] # holds jids that we already have in DB
 		self.AT_LEAST_ONE_DELETION_DONE = False
 		
@@ -199,16 +236,23 @@ class HistoryManager:
 	
 	def on_history_manager_window_delete_event(self, widget, event):
 		if self.AT_LEAST_ONE_DELETION_DONE:
+			def on_yes(clicked):
+				self.cur.execute('VACUUM')
+				self.con.commit()
+				gtk.main_quit()
+
+			def on_no():
+				gtk.main_quit()
+
 			dialog = dialogs.YesNoDialog(
 				_('Do you want to clean up the database? '
 				'(STRONGLY NOT RECOMMENDED IF GAJIM IS RUNNING)'),
 				_('Normally allocated database size will not be freed, '
 					'it will just become reusable. If you really want to reduce '
 					'database filesize, click YES, else click NO.'
-					'\n\nIn case you click YES, please wait...'))
-			if dialog.get_response() == gtk.RESPONSE_YES:
-				self.cur.execute('VACUUM')
-				self.con.commit()
+					'\n\nIn case you click YES, please wait...'),
+				on_response_yes=on_yes, on_response_no=on_no)
+			return
 							
 		gtk.main_quit()
 	
@@ -230,7 +274,7 @@ class HistoryManager:
 
 		self.logs_liststore.clear() # clear the store
 		
-		self.welcome_label.hide()
+		self.welcome_vbox.hide()
 		self.search_results_scrolledwindow.hide()
 		self.logs_scrolledwindow.show()
 
@@ -354,7 +398,7 @@ class HistoryManager:
 				if color:
 					message_ += ' foreground="%s"' % color
 				message_ += '>%s</span>' % \
-					gtkgui_helpers.escape_for_pango_markup(message)
+					gobject.markup_escape_text(message)
 				self.logs_liststore.append((log_line_id, jid_id, time_, message_,
 					subject, nickname))
 
@@ -416,8 +460,7 @@ class HistoryManager:
 		dlg = xml.get_widget('filechooserdialog')
 		dlg.set_title(_('Exporting History Logs...'))
 		dlg.set_current_folder(gajim.HOME_DIR)
-		if gtk.pygtk_version > (2, 8, 0):
-			dlg.props.do_overwrite_confirmation = True
+		dlg.props.do_overwrite_confirmation = True
 		response = dlg.run()
 		
 		if response == gtk.RESPONSE_OK: # user want us to export ;)
@@ -498,9 +541,8 @@ class HistoryManager:
 		if paths_len == 0: # nothing is selected
 			return
 
-		def on_ok(widget, liststore, list_of_paths):
+		def on_ok(liststore, list_of_paths):
 			# delete all rows from db that match jid_id
-			self.dialog.destroy()
 			list_of_rowrefs = []
 			for path in list_of_paths: # make them treerowrefs (it's needed)
 				list_of_rowrefs.append(gtk.TreeRowReference(liststore, path))
@@ -531,7 +573,7 @@ class HistoryManager:
 			'Do you really want to delete logs of the selected contact?',
 			'Do you really want to delete logs of the selected contacts?',
 			paths_len)
-		self.dialog = dialogs.ConfirmationDialog(pri_text,
+		dialogs.ConfirmationDialog(pri_text,
 			_('This is an irreversible operation.'), on_response_ok = (on_ok,
 			liststore, list_of_paths))
 
@@ -540,8 +582,7 @@ class HistoryManager:
 		if paths_len == 0: # nothing is selected
 			return
 
-		def on_ok(widget, liststore, list_of_paths):
-			self.dialog.destroy()
+		def on_ok(liststore, list_of_paths):
 			# delete rows from db that match log_line_id
 			list_of_rowrefs = []
 			for path in list_of_paths: # make them treerowrefs (it's needed)
@@ -567,7 +608,7 @@ class HistoryManager:
 		pri_text = i18n.ngettext(
 			'Do you really want to delete the selected message?',
 			'Do you really want to delete the selected messages?', paths_len)
-		self.dialog = dialogs.ConfirmationDialog(pri_text,
+		dialogs.ConfirmationDialog(pri_text,
 			_('This is an irreversible operation.'), on_response_ok = (on_ok,
 			liststore, list_of_paths))
 
@@ -576,7 +617,7 @@ class HistoryManager:
 		if text == '':
 			return
 
-		self.welcome_label.hide()
+		self.welcome_vbox.hide()
 		self.logs_scrolledwindow.hide()
 		self.search_results_scrolledwindow.show()
 		
@@ -618,3 +659,5 @@ if __name__ == '__main__':
 	signal.signal(signal.SIGINT, signal.SIG_DFL) # ^C exits the application
 	HistoryManager()
 	gtk.main()
+
+# vim: se ts=3:
