@@ -1,28 +1,35 @@
-##	message_window.py
+# -*- coding:utf-8 -*-
+## src/message_window.py
 ##
-## Copyright (C) 2003-2004 Yann Le Boulanger <asterix@lagaule.org>
-##                         Vincent Hanquez <tab@snarc.org>
-## Copyright (C) 2005 Yann Le Boulanger <asterix@lagaule.org>
-##                    Vincent Hanquez <tab@snarc.org>
-##                    Nikos Kouremenos <kourem@gmail.com>
-##                    Dimitur Kirov <dkirov@gmail.com>
-##                    Travis Shirk <travis@pobox.com>
-##                    Norman Rasmussen <norman@rasmussen.co.za>
-## Copyright (C) 2006 Travis Shirk <travis@pobox.com>
-## Copyright (C) 2006 Geobert Quach <geobert@gmail.com>
+## Copyright (C) 2003-2008 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2005-2008 Travis Shirk <travis AT pobox.com>
+##                         Nikos Kouremenos <kourem AT gmail.com>
+## Copyright (C) 2006 Geobert Quach <geobert AT gmail.com>
+##                    Dimitur Kirov <dkirov AT gmail.com>
+## Copyright (C) 2006-2008 Jean-Marie Traissard <jim AT lapin.org>
+## Copyright (C) 2007 Julien Pivotto <roidelapluie AT gmail.com>
+##                    Stephan Erb <steve-e AT h3c.de>
+## Copyright (C) 2008 Brendan Taylor <whateley AT gmail.com>
+##                    Jonathan Schleifer <js-gajim AT webkeks.org>
 ##
-## This program is free software; you can redistribute it and/or modify
+## This file is part of Gajim.
+##
+## Gajim is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 2 only.
+## by the Free Software Foundation; version 3 only.
 ##
-## This program is distributed in the hope that it will be useful,
+## Gajim is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ## GNU General Public License for more details.
 ##
+## You should have received a copy of the GNU General Public License
+## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+##
 
 import gtk
 import gobject
+import time
 
 import common
 import gtkgui_helpers
@@ -33,7 +40,7 @@ from common import gajim
 
 ####################
 
-class MessageWindow:
+class MessageWindow(object):
 	'''Class for windows which contain message like things; chats,
 	groupchats, etc.'''
 
@@ -47,32 +54,56 @@ class MessageWindow:
 		CLOSE_COMMAND,
 		CLOSE_CTRL_KEY
 	) = range(5)
-	
-	def __init__(self, acct, type):
-		# A dictionary of dictionaries where _contacts[account][jid] == A MessageControl
+
+	def __init__(self, acct, type_, parent_window=None, parent_paned=None):
+		# A dictionary of dictionaries
+		# where _contacts[account][jid] == A MessageControl
 		self._controls = {}
+
 		# If None, the window is not tied to any specific account
 		self.account = acct
 		# If None, the window is not tied to any specific type
-		self.type = type
+		self.type_ = type_
 		# dict { handler id: widget}. Keeps callbacks, which
 		# lead to cylcular references
 		self.handlers = {}
+		# Don't show warning dialogs when we want to delete the window
+		self.dont_warn_on_delete = False
 
 		self.widget_name = 'message_window'
 		self.xml = gtkgui_helpers.get_glade('%s.glade' % self.widget_name)
 		self.window = self.xml.get_widget(self.widget_name)
-		id = self.window.connect('delete-event', self._on_window_delete)
+		self.notebook = self.xml.get_widget('notebook')
+		self.parent_paned = None
+
+		if parent_window:
+			orig_window = self.window
+			self.window = parent_window
+			self.parent_paned = parent_paned
+			self.notebook.reparent(self.parent_paned)
+			self.parent_paned.pack2(self.notebook, resize=True, shrink=True)
+			orig_window.destroy()
+			del orig_window
+
+		# NOTE: we use 'connect_after' here because in 
+		# MessageWindowMgr._new_window we register handler that saves window
+		# state when closing it, and it should be called before
+		# MessageWindow._on_window_delete, which manually destroys window
+		# through win.destroy() - this means no additional handlers for
+		# 'delete-event' are called.
+		id = self.window.connect_after('delete-event', self._on_window_delete)
 		self.handlers[id] = self.window
 		id = self.window.connect('destroy', self._on_window_destroy)
 		self.handlers[id] = self.window
 		id = self.window.connect('focus-in-event', self._on_window_focus)
 		self.handlers[id] = self.window
 
-		keys=['<Control>h', '<Control>i', '<Control><Shift>Tab',
-				'<Control>Tab', '<Control>F4', '<Control>w',
-				'<Alt>Right', '<Alt>Left', '<Alt>c', 'Escape'] +\
-				['<Alt>'+str(i) for i in xrange(10)]
+		keys=['<Control>f', '<Control>g', '<Control>h', '<Control>i',
+			'<Control>l', '<Control>L', '<Control>n', '<Control>u', '<Control>v',
+			'<Control>b', '<Control><Shift>Tab', '<Control>Tab', '<Control>F4',
+			'<Control>w', '<Control>Page_Up', '<Control>Page_Down', '<Alt>Right',
+			'<Alt>Left', '<Alt>a', '<Alt>c', '<Alt>m', '<Alt>t', 'Escape'] + \
+			['<Alt>'+str(i) for i in xrange(10)]
 		accel_group = gtk.AccelGroup()
 		for key in keys:
 			keyval, mod = gtk.accelerator_parse(key)
@@ -85,7 +116,6 @@ class MessageWindow:
 		self.window.add_events(gtk.gdk.POINTER_MOTION_MASK)
 		self.alignment = self.xml.get_widget('alignment')
 
-		self.notebook = self.xml.get_widget('notebook')
 		id = self.notebook.connect('switch-page',
 			self._on_notebook_switch_page)
 		self.handlers[id] = self.notebook
@@ -107,14 +137,16 @@ class MessageWindow:
 		else:
 			nb_pos = gtk.POS_TOP
 		self.notebook.set_tab_pos(nb_pos)
-		if gajim.config.get('tabs_always_visible'):
+		window_mode = gajim.interface.msg_win_mgr.mode
+		if gajim.config.get('tabs_always_visible') or \
+		window_mode == MessageWindowMgr.ONE_MSG_WINDOW_ALWAYS_WITH_ROSTER:
 			self.notebook.set_show_tabs(True)
 			self.alignment.set_property('top-padding', 2)
 		else:
 			self.notebook.set_show_tabs(False)
 		self.notebook.set_show_border(gajim.config.get('tabs_border'))
 
-		# set up DnD if GTK+ version < 2.10, use OUR way to reorder tabs
+		# if GTK+ version < 2.10, use OUR way to reorder tabs (set up DnD)
 		if gtk.pygtk_version < (2, 10, 0) or gtk.gtk_version < (2, 10, 0):
 			self.hid = self.notebook.connect('drag_data_received',
 				self.on_tab_label_drag_data_received_cb)
@@ -123,9 +155,10 @@ class MessageWindow:
 				gtk.gdk.ACTION_MOVE)
 
 	def change_account_name(self, old_name, new_name):
-		if self._controls.has_key(old_name):
+		if old_name in self._controls:
 			self._controls[new_name] = self._controls[old_name]
 			del self._controls[old_name]
+
 		for ctrl in self.controls():
 			if ctrl.account == old_name:
 				ctrl.account = new_name
@@ -138,6 +171,9 @@ class MessageWindow:
 			n += len(dict)
 		return n
 
+	def resize(self, width, height):
+		gtkgui_helpers.resize_window(self.window, width, height)
+
 	def _on_window_focus(self, widget, event):
 		# window received focus, so if we had urgency REMOVE IT
 		# NOTE: we do not have to read the message (it maybe in a bg tab)
@@ -156,16 +192,40 @@ class MessageWindow:
 				self.redraw_tab(ctrl)
 
 	def _on_window_delete(self, win, event):
+		if self.dont_warn_on_delete:
+			# Destroy the window
+			return False
+
+		def on_yes(ctrl):
+			if self.on_delete_ok == 1:
+				self.dont_warn_on_delete = True
+				win.destroy()
+			self.on_delete_ok -= 1
+
+		def on_no(ctrl):
+			return
+
+		def on_minimize(ctrl):
+			ctrl.minimize()
+			if self.on_delete_ok == 1:
+				self.dont_warn_on_delete = True
+				win.destroy()
+			self.on_delete_ok -= 1
+
 		# Make sure all controls are okay with being deleted
+		ctrl_to_minimize = []
+		self.on_delete_ok = self.get_nb_controls()
 		for ctrl in self.controls():
-			if not ctrl.allow_shutdown(self.CLOSE_CLOSE_BUTTON):
-				return True # halt the delete
-		return False
+			ctrl.allow_shutdown(self.CLOSE_CLOSE_BUTTON, on_yes, on_no,
+				on_minimize)
+		return True # halt the delete for the moment
 
 	def _on_window_destroy(self, win):
 		for ctrl in self.controls():
 			ctrl.shutdown()
 		self._controls.clear()
+		# Clean up handlers connected to the parent window, this is important since
+		# self.window may be the RosterWindow
 		for i in self.handlers.keys():
 			if self.handlers[i].handler_is_connected(i):
 				self.handlers[i].disconnect(i)
@@ -173,9 +233,11 @@ class MessageWindow:
 		del self.handlers
 
 	def new_tab(self, control):
-		if not self._controls.has_key(control.account):
-			self._controls[control.account] = {}
 		fjid = control.get_full_jid()
+
+		if control.account not in self._controls:
+			self._controls[control.account] = {}
+
 		self._controls[control.account][fjid] = control
 
 		if self.get_num_controls() == 2:
@@ -194,11 +256,12 @@ class MessageWindow:
 		# Add notebook page and connect up to the tab's close button
 		xml = gtkgui_helpers.get_glade('message_window.glade', 'chat_tab_ebox')
 		tab_label_box = xml.get_widget('chat_tab_ebox')
-		widget =  xml.get_widget('tab_close_button')
+		widget = xml.get_widget('tab_close_button')
 		id = widget.connect('clicked', self._on_close_button_clicked, control)
 		control.handlers[id] = widget
-		
-		id = tab_label_box.connect('button-press-event', self.on_tab_eventbox_button_press_event, control.widget)
+
+		id = tab_label_box.connect('button-press-event', self.on_tab_eventbox_button_press_event,
+					control.widget)
 		control.handlers[id] = tab_label_box
 		self.notebook.append_page(control.widget, tab_label_box)
 
@@ -209,7 +272,10 @@ class MessageWindow:
 			self.setup_tab_dnd(control.widget)
 
 		self.redraw_tab(control)
-		self.window.show_all()
+		if self.parent_paned:
+			self.notebook.show_all()
+		else:
+			self.window.show_all()
 		# NOTE: we do not call set_control_active(True) since we don't know whether
 		# the tab is the active one.
 		self.show_title()
@@ -250,14 +316,40 @@ class MessageWindow:
 		if not control:
 			# No more control in this window
 			return
-		
+
 		# CTRL mask
 		if modifier & gtk.gdk.CONTROL_MASK:
-			if keyval == gtk.keysyms.h:
+			if keyval == gtk.keysyms.h: # CTRL + h
 				control._on_history_menuitem_activate()
 			elif control.type_id == message_control.TYPE_CHAT and \
-			keyval == gtk.keysyms.i:
+			keyval == gtk.keysyms.f: # CTRL + f
+				control._on_send_file_menuitem_activate(None)
+			elif control.type_id == message_control.TYPE_CHAT and \
+			keyval == gtk.keysyms.g: # CTRL + g
+				control._on_convert_to_gc_menuitem_activate(None)
+			elif control.type_id in (message_control.TYPE_CHAT,
+			message_control.TYPE_PM) and keyval == gtk.keysyms.i: # CTRL + i
 				control._on_contact_information_menuitem_activate(None)
+			elif keyval == gtk.keysyms.l or keyval == gtk.keysyms.L: # CTRL + l|L
+				control.conv_textview.clear()
+			elif control.type_id == message_control.TYPE_GC and \
+			keyval == gtk.keysyms.n: # CTRL + n
+				control._on_change_nick_menuitem_activate(None)
+			elif keyval == gtk.keysyms.u: # CTRL + u: emacs style clear line
+				control.clear(control.msg_textview)
+			elif keyval == gtk.keysyms.v: # CTRL + v: Paste into msg_textview
+				if not control.msg_textview.is_focus():
+					control.msg_textview.grab_focus()
+				# Paste into the msg textview
+				event = gtk.gdk.Event(gtk.gdk.KEY_PRESS)
+				event.window = self.window.window
+				event.time = int(time.time())
+				event.state = gtk.gdk.CONTROL_MASK
+				event.keyval = gtk.keysyms.v
+				control.msg_textview.emit('key_press_event', event)
+			elif control.type_id == message_control.TYPE_GC and \
+			keyval == gtk.keysyms.b: # CTRL + b
+				control._on_bookmark_room_menuitem_activate(None)
 			# Tab switch bindings
 			elif keyval == gtk.keysyms.ISO_Left_Tab: # CTRL + SHIFT + TAB
 				self.move_to_next_unread_tab(False)
@@ -265,15 +357,28 @@ class MessageWindow:
 				self.move_to_next_unread_tab(True)
 			elif keyval == gtk.keysyms.F4: # CTRL + F4
 				self.remove_tab(control, self.CLOSE_CTRL_KEY)
-			elif keyval == gtk.keysyms.w: # CTRL + W
-				self.remove_tab(control, self.CLOSE_CTRL_KEY)
+			elif keyval == gtk.keysyms.w: # CTRL + w
+				# CTRL + w removes latest word before sursor when User uses emacs
+				# theme
+				if not gtk.settings_get_default().get_property(
+				'gtk-key-theme-name') == 'Emacs':
+					self.remove_tab(control, self.CLOSE_CTRL_KEY)
+			elif keyval in (gtk.keysyms.Page_Up, gtk.keysyms.Page_Down):
+				# CTRL + PageUp | PageDown
+				# Create event and send it to notebook
+				event = gtk.gdk.Event(gtk.gdk.KEY_PRESS)
+				event.window = self.window.window
+				event.time = int(time.time())
+				event.state = gtk.gdk.CONTROL_MASK
+				event.keyval = int(keyval)
+				self.notebook.emit('key_press_event', event)
 
 		# MOD1 (ALT) mask
 		elif modifier & gtk.gdk.MOD1_MASK:
 			# Tab switch bindings
 			if keyval == gtk.keysyms.Right: # ALT + RIGHT
 				new = self.notebook.get_current_page() + 1
-				if new >= self.notebook.get_n_pages(): 
+				if new >= self.notebook.get_n_pages():
 					new = 0
 				self.notebook.set_current_page(new)
 			elif keyval == gtk.keysyms.Left: # ALT + LEFT
@@ -284,8 +389,14 @@ class MessageWindow:
 			elif chr(keyval) in st: # ALT + 1,2,3..
 				self.notebook.set_current_page(st.index(chr(keyval)))
 			elif keyval == gtk.keysyms.c: # ALT + C toggles chat buttons
-				control.chat_buttons_set_visible(not \
-					control.hide_chat_buttons_current)
+				control.chat_buttons_set_visible(not control.hide_chat_buttons)
+			elif keyval == gtk.keysyms.m: # ALT + M show emoticons menu
+				control.show_emoticons_menu()
+			elif keyval == gtk.keysyms.a: # ALT + A show actions menu
+				control.on_actions_button_clicked(control.actions_button)
+			elif control.type_id == message_control.TYPE_GC and \
+			keyval == gtk.keysyms.t: # ALT + t
+				control._on_change_subject_menuitem_activate(None)
 		# Close tab bindings
 		elif keyval == gtk.keysyms.Escape and \
 				gajim.config.get('escape_key_closes'): # Escape
@@ -295,7 +406,7 @@ class MessageWindow:
 		'''When close button is pressed: close a tab'''
 		self.remove_tab(control, self.CLOSE_CLOSE_BUTTON)
 
-	def show_title(self, urgent = True, control = None):
+	def show_title(self, urgent=True, control=None):
 		'''redraw the window's title'''
 		if not control:
 			control = self.get_active_control()
@@ -329,20 +440,24 @@ class MessageWindow:
 				name += '/' + control.resource
 
 		window_mode = gajim.interface.msg_win_mgr.mode
-
-		if self.get_num_controls() == 1:
-			label = name
-		elif window_mode == MessageWindowMgr.ONE_MSG_WINDOW_PERTYPE:
+		if window_mode == MessageWindowMgr.ONE_MSG_WINDOW_PERTYPE:
 			# Show the plural form since number of tabs > 1
-			if self.type == 'chat':
+			if self.type_ == 'chat':
 				label = _('Chats')
-			elif self.type == 'gc':
+			elif self.type_ == 'gc':
 				label = _('Group Chats')
 			else:
 				label = _('Private Chats')
+		elif window_mode == MessageWindowMgr.ONE_MSG_WINDOW_ALWAYS_WITH_ROSTER:
+			label = None
+		elif self.get_num_controls() == 1:
+			label = name
 		else:
 			label = _('Messages')
-		title = _('%s - Gajim') % label
+
+		title = 'Gajim'
+		if label:
+			title = '%s - %s' % (label, title)
 
 		if window_mode == MessageWindowMgr.ONE_MSG_WINDOW_PERACCT:
 			title = title + ": " + control.account
@@ -354,54 +469,90 @@ class MessageWindow:
 		else:
 			gtkgui_helpers.set_unset_urgency_hint(self.window, False)
 
-	def set_active_tab(self, jid, acct):
-		ctrl = self._controls[acct][jid]
+	def set_active_tab(self, ctrl):
 		ctrl_page = self.notebook.page_num(ctrl.widget)
 		self.notebook.set_current_page(ctrl_page)
-	
+		self.window.present()
+
 	def remove_tab(self, ctrl, method, reason = None, force = False):
 		'''reason is only for gc (offline status message)
 		if force is True, do not ask any confirmation'''
-		# Shutdown the MessageControl
-		if not force and not ctrl.allow_shutdown(method):
+		def close(ctrl):
+			if reason is not None: # We are leaving gc with a status message
+				ctrl.shutdown(reason)
+			else: # We are leaving gc without status message or it's a chat
+				ctrl.shutdown()
+			# Update external state
+			gajim.events.remove_events(ctrl.account, ctrl.get_full_jid,
+				types = ['printed_msg', 'chat', 'gc_msg'])
+
+			fjid = ctrl.get_full_jid()
+			jid = gajim.get_jid_without_resource(fjid)
+
+			fctrl = self.get_control(fjid, ctrl.account)
+			bctrl = self.get_control(jid, ctrl.account)
+			# keep last_message_time around unless this was our last control with
+			# that jid
+			if not fctrl and not bctrl and \
+			fjid in gajim.last_message_time[ctrl.account]:
+				del gajim.last_message_time[ctrl.account][fjid]
+
+			# Disconnect tab DnD only if GTK version < 2.10
+			if gtk.pygtk_version < (2, 10, 0) or gtk.gtk_version < (2, 10, 0):
+				self.disconnect_tab_dnd(ctrl.widget)
+
+			self.notebook.remove_page(self.notebook.page_num(ctrl.widget))
+
+			del self._controls[ctrl.account][fjid]
+
+			if len(self._controls[ctrl.account]) == 0:
+				del self._controls[ctrl.account]
+
+			self.check_tabs()
+			self.show_title()
+
+		def on_yes(ctrl):
+			close(ctrl)
+
+		def on_no(ctrl):
 			return
-		if reason is not None: # We are leaving gc with a status message
-			ctrl.shutdown(reason)
-		else: # We are leaving gc without status message or it's a chat
-			ctrl.shutdown()
 
-		# Update external state
-		gajim.events.remove_events(ctrl.account, ctrl.get_full_jid,
-			types = ['printed_msg', 'chat', 'gc_msg'])
-		del gajim.last_message_time[ctrl.account][ctrl.get_full_jid()]
+		def on_minimize(ctrl):
+			if method != self.CLOSE_COMMAND:
+				ctrl.minimize()
+				self.check_tabs()
+				return
+			close(ctrl)
 
-		# Disconnect tab DnD only if GTK version < 2.10
-		if gtk.pygtk_version < (2, 10, 0) or gtk.gtk_version < (2, 10, 0):
-			self.disconnect_tab_dnd(ctrl.widget)
-
-		self.notebook.remove_page(self.notebook.page_num(ctrl.widget))
-
-		fjid = ctrl.get_full_jid()
-		del self._controls[ctrl.account][fjid]
-		if len(self._controls[ctrl.account]) == 0:
-			del self._controls[ctrl.account]
+		# Shutdown the MessageControl
+		if force:
+			close(ctrl)
+		else:
+			ctrl.allow_shutdown(method, on_yes, on_no, on_minimize)
 
+	def check_tabs(self):
 		if self.get_num_controls() == 0:
 			# These are not called when the window is destroyed like this, fake it
 			gajim.interface.msg_win_mgr._on_window_delete(self.window, None)
 			gajim.interface.msg_win_mgr._on_window_destroy(self.window)
 			# dnd clean up
 			self.notebook.drag_dest_unset()
-			self.window.destroy()
+			if self.parent_paned:
+				# Don't close parent window, just remove the child
+				child = self.parent_paned.get_child2()
+				self.parent_paned.remove(child)
+			else:
+				self.window.destroy()
 			return # don't show_title, we are dead
 		elif self.get_num_controls() == 1: # we are going from two tabs to one
-			show_tabs_if_one_tab = gajim.config.get('tabs_always_visible')
+			window_mode = gajim.interface.msg_win_mgr.mode
+			show_tabs_if_one_tab = gajim.config.get('tabs_always_visible') or \
+				window_mode == MessageWindowMgr.ONE_MSG_WINDOW_ALWAYS_WITH_ROSTER
 			self.notebook.set_show_tabs(show_tabs_if_one_tab)
 			if not show_tabs_if_one_tab:
 				self.alignment.set_property('top-padding', 0)
-		self.show_title()
 
-			
+
 	def redraw_tab(self, ctrl, chatstate = None):
 		hbox = self.notebook.get_tab_label(ctrl.widget).get_children()[0]
 		status_img = hbox.get_children()[0]
@@ -486,35 +637,47 @@ class MessageWindow:
 			jid = key
 			try:
 				return self._controls[acct][jid]
-			except:
+			except Exception:
 				return None
 		else:
 			page_num = key
 			notebook = self.notebook
-			if page_num == None:
+			if page_num is None:
 				page_num = notebook.get_current_page()
 			nth_child = notebook.get_nth_page(page_num)
 			return self._widget_to_control(nth_child)
 
+	def has_control(self, jid, acct):
+		return (acct in self._controls and jid in self._controls[acct])
+
 	def change_key(self, old_jid, new_jid, acct):
-		'''Change the key of a control'''
+		'''Change the JID key of a control'''
 		try:
-			# Check if control exists
+			# Check if controls exists
 			ctrl = self._controls[acct][old_jid]
-		except:
+		except KeyError:
 			return
-		self._controls[acct][new_jid] = self._controls[acct][old_jid]
+
+		self._controls[acct][new_jid] = ctrl
 		del self._controls[acct][old_jid]
+
 		if old_jid in gajim.last_message_time[acct]:
 			gajim.last_message_time[acct][new_jid] = \
 				gajim.last_message_time[acct][old_jid]
 			del gajim.last_message_time[acct][old_jid]
 
 	def controls(self):
-		for ctrl_dict in self._controls.values():
-			for ctrl in ctrl_dict.values():
+		for jid_dict in self._controls.values():
+			for ctrl in jid_dict.values():
 				yield ctrl
 
+	def get_nb_controls(self):
+		nb_ctrl = 0
+		for jid_dict in self._controls.values():
+			for ctrl in jid_dict.values():
+				nb_ctrl += 1
+		return nb_ctrl
+
 	def move_to_next_unread_tab(self, forward):
 		ind = self.notebook.get_current_page()
 		current = ind
@@ -522,7 +685,7 @@ class MessageWindow:
 		first_composing_ind = -1 # id of first composing ctrl to switch to
 										# if no others controls have awaiting events
 		# loop until finding an unread tab or having done a complete cycle
-		while True: 
+		while True:
 			if forward == True: # look for the first unread tab on the right
 				ind = ind + 1
 				if ind >= self.notebook.get_n_pages():
@@ -570,17 +733,25 @@ class MessageWindow:
 		if old_no >= 0:
 			old_ctrl = self._widget_to_control(notebook.get_nth_page(old_no))
 			old_ctrl.set_control_active(False)
-		
+
 		new_ctrl = self._widget_to_control(notebook.get_nth_page(page_num))
 		new_ctrl.set_control_active(True)
 		self.show_title(control = new_ctrl)
 
-	def _on_notebook_key_press(self, widget, event):
 		control = self.get_active_control()
+		if isinstance(control, ChatControlBase):
+			control.msg_textview.grab_focus()
+
+	def _on_notebook_key_press(self, widget, event):
 		# Ctrl+PageUP / DOWN has to be handled by notebook
-		if event.state & gtk.gdk.CONTROL_MASK and event.keyval in (
-		gtk.keysyms.Page_Down, gtk.keysyms.Page_Up):
+		if (event.state & gtk.gdk.CONTROL_MASK and
+				event.keyval in (gtk.keysyms.Page_Down, gtk.keysyms.Page_Up)):
 			return False
+		# when tab itself is selected, make sure <- and -> are allowed for navigating between tabs
+		if event.keyval in (gtk.keysyms.Left, gtk.keysyms.Right):
+			return False
+
+		control = self.get_active_control()
 		if isinstance(control, ChatControlBase):
 			# we forwarded it to message textview
 			control.msg_textview.emit('key_press_event', event)
@@ -589,7 +760,7 @@ class MessageWindow:
 	def setup_tab_dnd(self, child):
 		'''Set tab label as drag source and connect the drag_data_get signal'''
 		tab_label = self.notebook.get_tab_label(child)
-		tab_label.dnd_handler = tab_label.connect('drag_data_get', 
+		tab_label.dnd_handler = tab_label.connect('drag_data_get',
 			self.on_tab_label_drag_data_get_cb)
 		self.handlers[tab_label.dnd_handler] = tab_label
 		tab_label.drag_source_set(gtk.gdk.BUTTON1_MASK, self.DND_TARGETS,
@@ -603,18 +774,18 @@ class MessageWindow:
 		selection.set(selection.target, 8, str(source_page_num))
 
 	def on_tab_label_drag_data_received_cb(self, widget, drag_context, x, y,
-		selection, type, time):
+		selection, type_, time):
 		'''Reorder the tabs according to the drop position'''
 		source_page_num = int(selection.data)
 		dest_page_num, to_right = self.get_tab_at_xy(x, y)
 		source_child = self.notebook.get_nth_page(source_page_num)
 		if dest_page_num != source_page_num:
 			self.notebook.reorder_child(source_child, dest_page_num)
-		
+
 	def get_tab_at_xy(self, x, y):
 		'''Thanks to Gaim
 		Return the tab under xy and
-		if its nearer from left or right side of the tab	
+		if its nearer from left or right side of the tab
 		'''
 		page_num = -1
 		to_right = False
@@ -635,7 +806,7 @@ class MessageWindow:
 				if (y >= tab_alloc.y) and \
 				(y <= (tab_alloc.y + tab_alloc.height)):
 					page_num = i
-				
+
 					if y > tab_alloc.y + (tab_alloc.height / 2.0):
 						to_right = True
 					break
@@ -659,36 +830,53 @@ class MessageWindow:
 		tab_label.disconnect(tab_label.dnd_handler)
 
 ################################################################################
-class MessageWindowMgr:
+class MessageWindowMgr(gobject.GObject):
 	'''A manager and factory for MessageWindow objects'''
+	__gsignals__ = {
+		'window-delete': (gobject.SIGNAL_RUN_LAST, None, (object,)),
+	}
 
 	# These constants map to common.config.opt_one_window_types indices
 	(
 	ONE_MSG_WINDOW_NEVER,
 	ONE_MSG_WINDOW_ALWAYS,
+	ONE_MSG_WINDOW_ALWAYS_WITH_ROSTER,
 	ONE_MSG_WINDOW_PERACCT,
-	ONE_MSG_WINDOW_PERTYPE
-	) = range(4)
-	# A key constant for the main window for all messages
+	ONE_MSG_WINDOW_PERTYPE,
+	) = range(5)
+	# A key constant for the main window in ONE_MSG_WINDOW_ALWAYS mode
 	MAIN_WIN = 'main'
+	# A key constant for the main window in ONE_MSG_WINDOW_ALWAYS_WITH_ROSTER mode
+	ROSTER_MAIN_WIN = 'roster'
 
-	def __init__(self):
+	def __init__(self, parent_window, parent_paned):
 		''' A dictionary of windows; the key depends on the config:
 		ONE_MSG_WINDOW_NEVER: The key is the contact JID
-		ONE_MSG_WINDOW_ALWAYS: The key is MessageWindowMgr.MAIN_WIN 
+		ONE_MSG_WINDOW_ALWAYS: The key is MessageWindowMgr.MAIN_WIN
+		ONE_MSG_WINDOW_ALWAYS_WITH_ROSTER: The key is MessageWindowMgr.MAIN_WIN
 		ONE_MSG_WINDOW_PERACCT: The key is the account name
 		ONE_MSG_WINDOW_PERTYPE: The key is a message type constant'''
+		gobject.GObject.__init__(self)
 		self._windows = {}
+
 		# Map the mode to a int constant for frequent compares
 		mode = gajim.config.get('one_message_window')
 		self.mode = common.config.opt_one_window_types.index(mode)
 
+		self.parent_win = parent_window
+		self.parent_paned = parent_paned
+
 	def change_account_name(self, old_name, new_name):
 		for win in self.windows():
 			win.change_account_name(old_name, new_name)
 
-	def _new_window(self, acct, type):
-		win = MessageWindow(acct, type)
+	def _new_window(self, acct, type_):
+		parent_win = None
+		parent_paned = None
+		if self.mode == self.ONE_MSG_WINDOW_ALWAYS_WITH_ROSTER:
+			parent_win = self.parent_win
+			parent_paned = self.parent_paned
+		win = MessageWindow(acct, type_, parent_win, parent_paned)
 		# we track the lifetime of this window
 		win.window.connect('delete-event', self._on_window_delete)
 		win.window.connect('destroy', self._on_window_destroy)
@@ -702,45 +890,52 @@ class MessageWindowMgr:
 
 	def get_window(self, jid, acct):
 		for win in self.windows():
-			if win.get_control(jid, acct):
+			if win.has_control(jid, acct):
 				return win
+
 		return None
 
 	def has_window(self, jid, acct):
-		return self.get_window(jid, acct) != None
+		return self.get_window(jid, acct) is not None
 
-	def one_window_opened(self, contact, acct, type):
+	def one_window_opened(self, contact=None, acct=None, type_=None):
 		try:
-			return self._windows[self._mode_to_key(contact, acct, type)] != None
+			return \
+				self._windows[self._mode_to_key(contact, acct, type_)] is not None
 		except KeyError:
 			return False
 
-	def _resize_window(self, win, acct, type):
+	def _resize_window(self, win, acct, type_):
 		'''Resizes window according to config settings'''
-		if not gajim.config.get('saveposition'):
-			return
-			
-		if self.mode == self.ONE_MSG_WINDOW_ALWAYS:
+		if self.mode in (self.ONE_MSG_WINDOW_ALWAYS,
+				self.ONE_MSG_WINDOW_ALWAYS_WITH_ROSTER):
 			size = (gajim.config.get('msgwin-width'),
 				gajim.config.get('msgwin-height'))
+			if self.mode == self.ONE_MSG_WINDOW_ALWAYS_WITH_ROSTER:
+				parent_size = win.window.get_size()
+				# Need to add the size of the now visible paned handle, otherwise
+				# the saved width of the message window decreases by this amount
+				handle_size = win.parent_paned.style_get_property('handle-size')
+				size = (parent_size[0] + size[0] + handle_size, size[1])
 		elif self.mode == self.ONE_MSG_WINDOW_PERACCT:
 			size = (gajim.config.get_per('accounts', acct, 'msgwin-width'),
 				gajim.config.get_per('accounts', acct, 'msgwin-height'))
 		elif self.mode in (self.ONE_MSG_WINDOW_NEVER, self.ONE_MSG_WINDOW_PERTYPE):
-			if type == message_control.TYPE_PM:
-				type = message_control.TYPE_CHAT
-			opt_width = type + '-msgwin-width'
-			opt_height = type + '-msgwin-height'
+			if type_ == message_control.TYPE_PM:
+				type_ = message_control.TYPE_CHAT
+			opt_width = type_ + '-msgwin-width'
+			opt_height = type_ + '-msgwin-height'
 			size = (gajim.config.get(opt_width), gajim.config.get(opt_height))
 		else:
 			return
+		win.resize(size[0], size[1])
+		if win.parent_paned:
+			win.parent_paned.set_position(parent_size[0])
 
-		gtkgui_helpers.resize_window(win.window, size[0], size[1])
-	
-	def _position_window(self, win, acct, type):
+	def _position_window(self, win, acct, type_):
 		'''Moves window according to config settings'''
-		if not gajim.config.get('saveposition') or\
-		self.mode == self.ONE_MSG_WINDOW_NEVER:
+		if (self.mode in [self.ONE_MSG_WINDOW_NEVER,
+		self.ONE_MSG_WINDOW_ALWAYS_WITH_ROSTER]):
 			return
 
 		if self.mode == self.ONE_MSG_WINDOW_ALWAYS:
@@ -750,57 +945,64 @@ class MessageWindowMgr:
 			pos = (gajim.config.get_per('accounts', acct, 'msgwin-x-position'),
 				gajim.config.get_per('accounts', acct, 'msgwin-y-position'))
 		elif self.mode == self.ONE_MSG_WINDOW_PERTYPE:
-			pos = (gajim.config.get(type + '-msgwin-x-position'),
-				gajim.config.get(type + '-msgwin-y-position'))
+			pos = (gajim.config.get(type_ + '-msgwin-x-position'),
+				gajim.config.get(type_ + '-msgwin-y-position'))
 		else:
 			return
 
 		gtkgui_helpers.move_window(win.window, pos[0], pos[1])
 
-	def _mode_to_key(self, contact, acct, type, resource = None):
+	def _mode_to_key(self, contact, acct, type_, resource = None):
 		if self.mode == self.ONE_MSG_WINDOW_NEVER:
 			key = acct + contact.jid
 			if resource:
 				key += '/' + resource
+			return key
 		elif self.mode == self.ONE_MSG_WINDOW_ALWAYS:
-			key = self.MAIN_WIN
+			return self.MAIN_WIN
+		elif self.mode == self.ONE_MSG_WINDOW_ALWAYS_WITH_ROSTER:
+			return self.ROSTER_MAIN_WIN
 		elif self.mode == self.ONE_MSG_WINDOW_PERACCT:
-			key = acct
+			return acct
 		elif self.mode == self.ONE_MSG_WINDOW_PERTYPE:
-			key = type
-		return key
+			return type_
 
-	def create_window(self, contact, acct, type, resource = None):
-		key = None
+	def create_window(self, contact, acct, type_, resource = None):
 		win_acct = None
 		win_type = None
-		win_role = 'messages'
+		win_role = None # X11 window role
 
-		key = self._mode_to_key(contact, acct, type, resource)
+		win_key = self._mode_to_key(contact, acct, type_, resource)
 		if self.mode == self.ONE_MSG_WINDOW_PERACCT:
 			win_acct = acct
 			win_role = acct
 		elif self.mode == self.ONE_MSG_WINDOW_PERTYPE:
-			win_type = type
-			win_role = type
+			win_type = type_
+			win_role = type_
 		elif self.mode == self.ONE_MSG_WINDOW_NEVER:
-			win_type = type
+			win_type = type_
 			win_role = contact.jid
+		elif self.mode == self.ONE_MSG_WINDOW_ALWAYS:
+			win_role = 'messages'
 
 		win = None
 		try:
-			win = self._windows[key]
+			win = self._windows[win_key]
 		except KeyError:
 			win = self._new_window(win_acct, win_type)
 
-		win.window.set_role(win_role)
+		if win_role:
+			win.window.set_role(win_role)
 
 		# Position and size window based on saved state and window mode
-		if not self.one_window_opened(contact, acct, type):
-			self._position_window(win, acct, type)
-			self._resize_window(win, acct, type)
+		if not self.one_window_opened(contact, acct, type_):
+			if gajim.config.get('msgwin-max-state'):
+				win.window.maximize()
+			else:
+				self._resize_window(win, acct, type_)
+				self._position_window(win, acct, type_)
 
-		self._windows[key] = win
+		self._windows[win_key] = win
 		return win
 
 	def change_key(self, old_jid, new_jid, acct):
@@ -822,6 +1024,7 @@ class MessageWindowMgr:
 	def _on_window_destroy(self, win):
 		for k in self._windows.keys():
 			if self._windows[k].window == win:
+				self.emit('window-delete', self._windows[k])
 				del self._windows[k]
 				return
 
@@ -832,12 +1035,20 @@ class MessageWindowMgr:
 			return win.get_control(jid, acct)
 		return None
 
-	def get_controls(self, type = None, acct = None):
+	def get_gc_control(self, jid, acct):
+		'''Same as get_control. Was briefly required, is not any more.
+May be useful some day in the future?'''
+		ctrl = self.get_control(jid, acct)
+		if ctrl and ctrl.type_id == message_control.TYPE_GC:
+			return ctrl
+		return None
+
+	def get_controls(self, type_=None, acct=None):
 		ctrls = []
 		for c in self.controls():
 			if acct and c.account != acct:
 				continue
-			if not type or c.type_id == type:
+			if not type_ or c.type_id == type_:
 				ctrls.append(c)
 		return ctrls
 
@@ -850,18 +1061,18 @@ class MessageWindowMgr:
 			for c in w.controls():
 				yield c
 
-	def shutdown(self):
+	def shutdown(self, width_adjust=0):
 		for w in self.windows():
-			self.save_state(w)
-			w.window.hide()
-			w.window.destroy()
+			self.save_state(w, width_adjust)
+			if not w.parent_paned:
+				w.window.hide()
+				w.window.destroy()
+
 		gajim.interface.save_config()
 
-	def save_state(self, msg_win):
-		if not gajim.config.get('saveposition'):
-			return
-		
+	def save_state(self, msg_win, width_adjust=0):
 		# Save window size and position
+		max_win_key = 'msgwin-max-state'
 		pos_x_key = 'msgwin-x-position'
 		pos_y_key = 'msgwin-y-position'
 		size_width_key = 'msgwin-width'
@@ -878,15 +1089,18 @@ class MessageWindowMgr:
 		elif self.mode == self.ONE_MSG_WINDOW_PERACCT:
 			acct = msg_win.account
 		elif self.mode == self.ONE_MSG_WINDOW_PERTYPE:
-			type = msg_win.type
-			pos_x_key = type + '-msgwin-x-position'
-			pos_y_key = type + '-msgwin-y-position'
-			size_width_key = type + '-msgwin-width'
-			size_height_key = type + '-msgwin-height'
+			type_ = msg_win.type_
+			pos_x_key = type_ + '-msgwin-x-position'
+			pos_y_key = type_ + '-msgwin-y-position'
+			size_width_key = type_ + '-msgwin-width'
+			size_height_key = type_ + '-msgwin-height'
 		elif self.mode == self.ONE_MSG_WINDOW_NEVER:
-			type = msg_win.type
-			size_width_key = type + '-msgwin-width'
-			size_height_key = type + '-msgwin-height'
+			type_ = msg_win.type_
+			size_width_key = type_ + '-msgwin-width'
+			size_height_key = type_ + '-msgwin-height'
+		elif self.mode == self.ONE_MSG_WINDOW_ALWAYS_WITH_ROSTER:
+			# Ignore any hpaned width
+			width = msg_win.notebook.allocation.width
 
 		if acct:
 			gajim.config.set_per('accounts', acct, size_width_key, width)
@@ -895,11 +1109,15 @@ class MessageWindowMgr:
 			if self.mode != self.ONE_MSG_WINDOW_NEVER:
 				gajim.config.set_per('accounts', acct, pos_x_key, x)
 				gajim.config.set_per('accounts', acct, pos_y_key, y)
-		
+
 		else:
+			win_maximized = msg_win.window.window.get_state() == \
+				gtk.gdk.WINDOW_STATE_MAXIMIZED
+			gajim.config.set(max_win_key, win_maximized)
+			width += width_adjust
 			gajim.config.set(size_width_key, width)
 			gajim.config.set(size_height_key, height)
-			
+
 			if self.mode != self.ONE_MSG_WINDOW_NEVER:
 				gajim.config.set(pos_x_key, x)
 				gajim.config.set(pos_y_key, y)
@@ -908,7 +1126,6 @@ class MessageWindowMgr:
 		for w in self.windows():
 			self.save_state(w)
 		gajim.interface.save_config()
-		# Map the mode to a int constant for frequent compares
 		mode = gajim.config.get('one_message_window')
 		if self.mode == common.config.opt_one_window_types.index(mode):
 			# No change
@@ -917,17 +1134,34 @@ class MessageWindowMgr:
 
 		controls = []
 		for w in self.windows():
-			w.window.hide()
+			# Note, we are taking care not to hide/delete the roster window when the
+			# MessageWindow is embedded.
+			if not w.parent_paned:
+				w.window.hide()
+			else:
+				# Stash current size so it can be restored if the MessageWindow
+				# is not longer embedded
+				roster_width = w.parent_paned.get_child1().allocation.width
+				gajim.config.set('roster_width', roster_width)
+
 			while w.notebook.get_n_pages():
 				page = w.notebook.get_nth_page(0)
 				ctrl = w._widget_to_control(page)
 				w.notebook.remove_page(0)
 				page.unparent()
 				controls.append(ctrl)
-			# Must clear _controls from window to prevent
-			# MessageControl.shutdown calls
+
+			# Must clear _controls to prevent MessageControl.shutdown calls
 			w._controls = {}
-			w.window.destroy()
+			if not w.parent_paned:
+				w.window.destroy()
+			else:
+				# Don't close parent window, just remove the child
+				child = w.parent_paned.get_child2()
+				w.parent_paned.remove(child)
+				gtkgui_helpers.resize_window(w.window,
+					gajim.config.get('roster_width'),
+					gajim.config.get('roster_height'))
 
 		self._windows = {}
 
@@ -938,3 +1172,5 @@ class MessageWindowMgr:
 							ctrl.type_id)
 			ctrl.parent_win = mw
 			mw.new_tab(ctrl)
+
+# vim: se ts=3:
