@@ -1,19 +1,30 @@
-##	remote_control.py
+# -*- coding:utf-8 -*-
+## src/remote_control.py
 ##
-## Copyright (C) 2005-2006 Yann Le Boulanger <asterix@lagaule.org>
-## Copyright (C) 2005-2006 Nikos Kouremenos <kourem@gmail.com>
-## Copyright (C) 2005-2006 Dimitur Kirov <dkirov@gmail.com>
-## Copyright (C) 2005-2006 Andrew Sayman <lorien420@myrealbox.com>
+## Copyright (C) 2005-2006 Andrew Sayman <lorien420 AT myrealbox.com>
+##                         Dimitur Kirov <dkirov AT gmail.com>
+##                         Nikos Kouremenos <kourem AT gmail.com>
+## Copyright (C) 2005-2008 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2006-2007 Travis Shirk <travis AT pobox.com>
+## Copyright (C) 2006-2008 Jean-Marie Traissard <jim AT lapin.org>
+## Copyright (C) 2007 Lukas Petrovicky <lukas AT petrovicky.net>
+##                    Julien Pivotto <roidelapluie AT gmail.com>
+## Copyright (C) 2008 Jonathan Schleifer <js-gajim AT webkeks.org>
 ##
-## This program is free software; you can redistribute it and/or modify
+## This file is part of Gajim.
+##
+## Gajim is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 2 only.
+## by the Free Software Foundation; version 3 only.
 ##
-## This program is distributed in the hope that it will be useful,
+## Gajim is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ## GNU General Public License for more details.
 ##
+## You should have received a copy of the GNU General Public License
+## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+##
 
 import gobject
 import os
@@ -91,7 +102,10 @@ class Remote:
 
 	def raise_signal(self, signal, arg):
 		if self.signal_object:
-			getattr(self.signal_object, signal)(get_dbus_struct(arg))
+			try:
+				getattr(self.signal_object, signal)(get_dbus_struct(arg))
+			except UnicodeDecodeError:
+				pass # ignore error when we fail to announce on dbus
 
 
 class SignalObject(dbus.service.Object):
@@ -121,6 +135,10 @@ class SignalObject(dbus.service.Object):
 	def ContactAbsence(self, account_and_array):
 		pass
 
+	@dbus.service.signal(INTERFACE, signature='av')
+	def ContactStatus(self, account_and_array):
+		pass
+
 	@dbus.service.signal(INTERFACE, signature='av')
 	def NewMessage(self, account_and_array):
 		pass
@@ -223,6 +241,31 @@ class SignalObject(dbus.service.Object):
 
 		return connected_account, contact
 
+	def _get_account_for_groupchat(self, account, room_jid):
+		'''get the account which is connected to groupchat (if not given)
+		or check if the given account is connected to the groupchat'''
+		connected_account = None
+		accounts = gajim.contacts.get_accounts()
+		# if there is only one account in roster, take it as default
+		# if user did not ask for account
+		if not account and len(accounts) == 1:
+			account = accounts[0]
+		if account:
+			if gajim.connections[account].connected > 1 and \
+			room_jid in gajim.gc_connected[account] and \
+			gajim.gc_connected[account][room_jid]:
+				# account and groupchat are connected
+				connected_account = account
+		else:
+			for account in accounts:
+				if gajim.connections[account].connected > 1 and \
+				room_jid in gajim.gc_connected[account] and \
+				gajim.gc_connected[account][room_jid]:
+					# account and groupchat are connected
+					connected_account = account
+					break
+		return connected_account
+
 	@dbus.service.method(INTERFACE, in_signature='sss', out_signature='b')
 	def send_file(self, file_path, jid, account):
 		'''send file, located at 'file_path' to 'jid', using account 
@@ -231,7 +274,7 @@ class SignalObject(dbus.service.Object):
 		connected_account, contact = self._get_account_and_contact(account, jid)
 
 		if connected_account:
-			if file_path[:7] == 'file://':
+			if file_path.startswith('file://'):
 				file_path=file_path[7:]
 			if os.path.isfile(file_path): # is it file?
 				gajim.interface.instances['file_transfers'].send_file(
@@ -239,7 +282,7 @@ class SignalObject(dbus.service.Object):
 				return DBUS_BOOLEAN(True)
 		return DBUS_BOOLEAN(False)
 
-	def _send_message(self, jid, message, keyID, account, type = 'chat',
+	def _send_message(self, jid, message, keyID, account, type_ = 'chat',
 	subject = None):
 		'''can be called from send_chat_message (default when send_message)
 		or send_single_message'''
@@ -251,7 +294,7 @@ class SignalObject(dbus.service.Object):
 		connected_account, contact = self._get_account_and_contact(account, jid)
 		if connected_account:
 			connection = gajim.connections[connected_account]
-			connection.send_message(jid, message, keyID, type, subject)
+			connection.send_message(jid, message, keyID, type_, subject)
 			return DBUS_BOOLEAN(True)
 		return DBUS_BOOLEAN(False)
 
@@ -269,6 +312,19 @@ class SignalObject(dbus.service.Object):
 		jid = self._get_real_jid(jid, account)
 		return self._send_message(jid, message, keyID, account, type, subject)
 
+	@dbus.service.method(INTERFACE, in_signature='sss', out_signature='b')
+	def send_groupchat_message(self, room_jid, message, account):
+		'''Send 'message' to groupchat 'room_jid',
+		using account (optional) 'account'.'''
+		if not room_jid or not message:
+			return DBUS_BOOLEAN(False)
+		connected_account = self._get_account_for_groupchat(account, room_jid)
+		if connected_account:
+			connection = gajim.connections[connected_account]
+			connection.send_gc_message(room_jid, message)
+			return DBUS_BOOLEAN(True)
+		return DBUS_BOOLEAN(False)
+
 	@dbus.service.method(INTERFACE, in_signature='ss', out_signature='b')
 	def open_chat(self, jid, account):
 		'''Shows the tabbed window for new message to 'jid', using account
@@ -279,7 +335,7 @@ class SignalObject(dbus.service.Object):
 		jid = self._get_real_jid(jid, account)
 		try:
 			jid = helpers.parse_jid(jid)
-		except:
+		except Exception:
 			# Jid is not conform, ignore it
 			return DBUS_BOOLEAN(False)
 
@@ -313,7 +369,7 @@ class SignalObject(dbus.service.Object):
 			connected_account = first_connected_acct
 
 		if connected_account:
-			gajim.interface.roster.new_chat_from_jid(connected_account, jid)
+			gajim.interface.new_chat_from_jid(connected_account, jid)
 			# preserve the 'steal focus preservation'
 			win = gajim.interface.msg_win_mgr.get_window(jid,
 				connected_account).window
@@ -328,7 +384,6 @@ class SignalObject(dbus.service.Object):
 		if not specified status is changed for all accounts. '''
 		if status not in ('offline', 'online', 'chat', 
 			'away', 'xa', 'dnd', 'invisible'):
-			raise InvalidArgument
 			return DBUS_BOOLEAN(False)
 		if account:
 			gobject.idle_add(gajim.interface.roster.send_status, account,
@@ -381,7 +436,7 @@ class SignalObject(dbus.service.Object):
 	def account_info(self, account):
 		'''show info on account: resource, jid, nick, prio, message'''
 		result = DBUS_DICT_SS()
-		if gajim.connections.has_key(account):
+		if account in gajim.connections:
 			# account is valid
 			con = gajim.connections[account]
 			index = con.connected
@@ -410,7 +465,7 @@ class SignalObject(dbus.service.Object):
 			if acct in accounts:
 				for jid in gajim.contacts.get_jid_list(acct):
 					item = self._contacts_as_dbus_structure(
-						gajim.contacts.get_contact(acct, jid))
+						gajim.contacts.get_contacts(acct, jid))
 					if item:
 						result.append(item)
 		return result
@@ -429,6 +484,19 @@ class SignalObject(dbus.service.Object):
 			else:
 				win.window.focus(long(time()))
 
+	@dbus.service.method(INTERFACE, in_signature='', out_signature='')
+	def toggle_ipython(self):
+		''' shows/hides the ipython window '''
+		win = gajim.ipython_window
+		if win:
+			if win.window.is_visible():
+				gobject.idle_add(win.hide)
+			else:
+				win.show_all()
+				win.present()
+		else:
+			gajim.interface.create_ipython_window()
+
 	@dbus.service.method(INTERFACE, in_signature='', out_signature='a{ss}')
 	def prefs_list(self):
 		prefs_dict = DBUS_DICT_SS()
@@ -503,7 +571,7 @@ class SignalObject(dbus.service.Object):
 			accounts = [account]
 		contact_exists = False
 		for account in accounts:
-			contacts = gajim.contacts.get_contact(account, jid)
+			contacts = gajim.contacts.get_contacts(account, jid)
 			if contacts:
 				gajim.connections[account].unsubscribe(jid)
 				for contact in contacts:
@@ -531,12 +599,12 @@ class SignalObject(dbus.service.Object):
 		nick_in_roster = None # Is jid a nick ?
 		for account in accounts:
 			# Does jid exists in roster of one account ?
-			if gajim.contacts.get_contacts_from_jid(account, jid):
+			if gajim.contacts.get_contacts(account, jid):
 				return jid
 			if not nick_in_roster:
 				# look in all contact if one has jid as nick
 				for jid_ in gajim.contacts.get_jid_list(account):
-					c = gajim.contacts.get_contacts_from_jid(account, jid_)
+					c = gajim.contacts.get_contacts(account, jid_)
 					if c[0].name == jid:
 						nick_in_roster = jid_
 						break
@@ -552,7 +620,7 @@ class SignalObject(dbus.service.Object):
 			return None
 		prim_contact = None # primary contact
 		for contact in contacts:
-			if prim_contact == None or contact.priority > prim_contact.priority:
+			if prim_contact is None or contact.priority > prim_contact.priority:
 				prim_contact = contact
 		contact_dict = DBUS_DICT_SV()
 		contact_dict['name'] = DBUS_STRING(prim_contact.name)
@@ -571,6 +639,9 @@ class SignalObject(dbus.service.Object):
 			resource_props = dbus.Struct((DBUS_STRING(contact.resource),
 				dbus.Int32(contact.priority), DBUS_STRING(contact.status)))
 			contact_dict['resources'].append(resource_props)
+		contact_dict['groups'] = dbus.Array([], signature='(s)')
+		for group in prim_contact.groups:
+			contact_dict['groups'].append((DBUS_STRING(group),))
 		return contact_dict
 
 	@dbus.service.method(INTERFACE, in_signature='', out_signature='s')
@@ -588,10 +659,10 @@ class SignalObject(dbus.service.Object):
 	@dbus.service.method(INTERFACE, in_signature='ss', out_signature='')
 	def send_xml(self, xml, account):
 		if account:
-			gajim.connections[account].send_stanza(xml)
+			gajim.connections[account].send_stanza(str(xml))
 		else:
 			for acc in gajim.contacts.get_accounts():
-				gajim.connections[acc].send_stanza(xml)
+				gajim.connections[acc].send_stanza(str(xml))
 
 	@dbus.service.method(INTERFACE, in_signature='ssss', out_signature='')
 	def join_room(self, room_jid, nick, password, account):
@@ -609,4 +680,6 @@ class SignalObject(dbus.service.Object):
 			gajim.interface.instances[account]['join_gc'] = \
 					JoinGroupchatWindow(account, room_jid, nick)
 		else:
-			gajim.connections[account].join_gc(nick, room_jid, password)
+			gajim.interface.join_gc_room(account, room_jid, nick, password)
+
+# vim: se ts=3:
