@@ -1,18 +1,33 @@
-##	vcard.py (has VcardWindow class and a func get_avatar_pixbuf_encoded_mime)
+# -*- coding:utf-8 -*-
+## src/vcard.py
 ##
-## Copyright (C) 2003-2006 Yann Le Boulanger <asterix@lagaule.org>
-## Copyright (C) 2005-2006 Nikos Kouremenos <kourem@gmail.com>
-## Copyright (C) 2006 Stefan Bethge <stefan@lanpartei.de>
+## Copyright (C) 2003-2008 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2005 Vincent Hanquez <tab AT snarc.org>
+## Copyright (C) 2005-2006 Nikos Kouremenos <kourem AT gmail.com>
+## Copyright (C) 2006 Junglecow J <junglecow AT gmail.com>
+##                    Dimitur Kirov <dkirov AT gmail.com>
+##                    Travis Shirk <travis AT pobox.com>
+##                    Stefan Bethge <stefan AT lanpartei.de>
+## Copyright (C) 2006-2008 Jean-Marie Traissard <jim AT lapin.org>
+## Copyright (C) 2007 Lukas Petrovicky <lukas AT petrovicky.net>
+## Copyright (C) 2008 Brendan Taylor <whateley AT gmail.com>
+##                    Jonathan Schleifer <js-gajim AT webkeks.org>
+##                    Stephan Erb <steve-e AT h3c.de>
 ##
-## This program is free software; you can redistribute it and/or modify
+## This file is part of Gajim.
+##
+## Gajim is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
-## by the Free Software Foundation; version 2 only.
+## by the Free Software Foundation; version 3 only.
 ##
-## This program is distributed in the hope that it will be useful,
+## Gajim is distributed in the hope that it will be useful,
 ## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 ## GNU General Public License for more details.
 ##
+## You should have received a copy of the GNU General Public License
+## along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+##
 
 # THIS FILE IS FOR **OTHERS'** PROFILE (when we VIEW their INFO)
 
@@ -21,6 +36,7 @@ import gobject
 import base64
 import time
 import locale
+import os
 
 import gtkgui_helpers
 
@@ -36,15 +52,15 @@ def get_avatar_pixbuf_encoded_mime(photo):
 	img_decoded = None
 	avatar_encoded = None
 	avatar_mime_type = None
-	if photo.has_key('BINVAL'):
+	if 'BINVAL' in photo:
 		img_encoded = photo['BINVAL']
 		avatar_encoded = img_encoded
 		try:
 			img_decoded = base64.decodestring(img_encoded)
-		except:
+		except Exception:
 			pass
 	if img_decoded:
-		if photo.has_key('TYPE'):
+		if 'TYPE' in photo:
 			avatar_mime_type = photo['TYPE']
 			pixbuf = gtkgui_helpers.get_pixbuf_from_data(img_decoded)
 		else:
@@ -69,7 +85,10 @@ class VcardWindow:
 
 		# Get real jid
 		if gc_contact:
-			if gc_contact.jid:
+			# Don't use real jid if room is (semi-)anonymous
+			gc_control = gajim.interface.msg_win_mgr.get_gc_control(
+			gc_contact.room_jid, account)
+			if gc_contact.jid and not gc_control.is_anonymous:
 				self.real_jid = gc_contact.jid
 				if gc_contact.resource:
 					self.real_jid += '/' + gc_contact.resource
@@ -78,6 +97,17 @@ class VcardWindow:
 		else:
 			self.real_jid = contact.get_full_jid()
 
+		puny_jid = helpers.sanitize_filename(contact.jid)
+		local_avatar_basepath = os.path.join(gajim.AVATAR_PATH, puny_jid) + \
+			'_local'
+		for extension in ('.png', '.jpeg'):
+			local_avatar_path = local_avatar_basepath + extension
+			if os.path.isfile(local_avatar_path):
+				image = self.xml.get_widget('custom_avatar_image')
+				image.set_from_file(local_avatar_path)
+				image.show()
+				self.xml.get_widget('custom_avatar_label').show()
+				break
 		self.avatar_mime_type = None
 		self.avatar_encoded = None
 		self.vcard_arrived = False
@@ -111,27 +141,10 @@ class VcardWindow:
 			connection.annotations[self.contact.jid] = annotation
 			connection.store_annotations()
 
-
 	def on_vcard_information_window_key_press_event(self, widget, event):
 		if event.keyval == gtk.keysyms.Escape:
 			self.window.destroy()
 
-	def on_log_history_checkbutton_toggled(self, widget):
-		#log conversation history?
-		oldlog = True
-		no_log_for = gajim.config.get_per('accounts', self.account,
-			'no_log_for').split()
-		if self.contact.jid in no_log_for:
-			oldlog = False
-		log = widget.get_active()
-		if not log and not self.contact.jid in no_log_for:
-			no_log_for.append(self.contact.jid)
-		if log and self.contact.jid in no_log_for:
-			no_log_for.remove(self.contact.jid)
-		if oldlog != log:
-			gajim.config.set_per('accounts', self.account, 'no_log_for',
-				' '.join(no_log_for))
-
 	def on_PHOTO_eventbox_button_press_event(self, widget, event):
 		'''If right-clicked, show popup'''
 		if event.button == 3: # right click
@@ -152,8 +165,10 @@ class VcardWindow:
 			if value and entry_name == 'URL_label':
 				if gtk.pygtk_version >= (2, 10, 0) and gtk.gtk_version >= (2, 10, 0):
 					widget = gtk.LinkButton(value, value)
+					widget.set_alignment(0, 0)
 				else:
 					widget = gtk.Label(value)
+					widget.set_alignment(0, 0)
 					widget.set_selectable(True)
 				widget.show()
 				table = self.xml.get_widget('personal_info_table')
@@ -170,6 +185,8 @@ class VcardWindow:
 				pixbuf, self.avatar_encoded, self.avatar_mime_type = \
 					get_avatar_pixbuf_encoded_mime(vcard[i])
 				image = self.xml.get_widget('PHOTO_image')
+				image.show()
+				self.xml.get_widget('user_avatar_label').show()
 				if not pixbuf:
 					image.set_from_icon_name('stock_person',
 						gtk.ICON_SIZE_DIALOG)
@@ -212,7 +229,7 @@ class VcardWindow:
 		i = 0
 		client = ''
 		os = ''
-		while self.os_info.has_key(i):
+		while i in self.os_info:
 			if not self.os_info[i]['resource'] or \
 					self.os_info[i]['resource'] == resource:
 				self.os_info[i]['client'] = client_info
@@ -236,7 +253,7 @@ class VcardWindow:
 	def fill_status_label(self):
 		if self.xml.get_widget('information_notebook').get_n_pages() < 5:
 			return
-		contact_list = gajim.contacts.get_contact(self.account, self.contact.jid)
+		contact_list = gajim.contacts.get_contacts(self.account, self.contact.jid)
 		connected_contact_list = []
 		for c in contact_list:
 			if c.show not in ('offline', 'error'):
@@ -287,11 +304,11 @@ class VcardWindow:
 		subscription_label = self.xml.get_widget('subscription_label')
 		ask_label = self.xml.get_widget('ask_label')
 		if self.gc_contact:
-			self.xml.get_widget('subscription_title_label').set_text(_("Role:"))
+			self.xml.get_widget('subscription_title_label').set_markup(_("<b>Role:</b>"))
 			uf_role = helpers.get_uf_role(self.gc_contact.role)
 			subscription_label.set_text(uf_role)
 
-			self.xml.get_widget('ask_title_label').set_text(_("Affiliation:"))
+			self.xml.get_widget('ask_title_label').set_markup(_("<b>Affiliation:</b>"))
 			uf_affiliation = helpers.get_uf_affiliation(self.gc_contact.affiliation)
 			ask_label.set_text(uf_affiliation)
 		else:
@@ -312,17 +329,11 @@ class VcardWindow:
 			ask_label.set_text(uf_ask)
 			eb = self.xml.get_widget('ask_label_eventbox')
 			if self.contact.ask == 'subscribe':
-				tooltips.set_tip(eb,
-				_("You are waiting contact's answer about your subscription request"))
+				tt_text = _("You are waiting contact's answer about your subscription request")
+			else:
+				tt_text = _("There is no pending subscription request.")
+			tooltips.set_tip(eb, tt_text)
 
-		log = True
-		if self.contact.jid in gajim.config.get_per('accounts', self.account,
-			'no_log_for').split(' '):
-			log = False
-		checkbutton = self.xml.get_widget('log_history_checkbutton')
-		checkbutton.set_active(log)
-		checkbutton.connect('toggled', self.on_log_history_checkbutton_toggled)
-		
 		resources = '%s (%s)' % (self.contact.resource, unicode(
 			self.contact.priority))
 		uf_resources = self.contact.resource + _(' resource with priority ')\
@@ -330,14 +341,15 @@ class VcardWindow:
 		if not self.contact.status:
 			self.contact.status = ''
 
-		# Request list time status
-		if self.gc_contact:
-			j, r = gajim.get_room_and_nick_from_fjid(self.real_jid)
-			gajim.connections[self.account].request_last_status_time(j, r,
-				self.contact.jid)
-		else:
-			gajim.connections[self.account].request_last_status_time(
-				self.contact.jid, self.contact.resource)
+		# Request list time status only if contact is offline
+		if self.contact.show == 'offline':
+			if self.gc_contact:
+				j, r = gajim.get_room_and_nick_from_fjid(self.real_jid)
+				gajim.connections[self.account].request_last_status_time(j, r,
+					self.contact.jid)
+			else:
+				gajim.connections[self.account].request_last_status_time(
+					self.contact.jid, self.contact.resource)
 
 		# do not wait for os_info if contact is not connected or has error
 		# additional check for observer is needed, as show is offline for him
@@ -355,7 +367,7 @@ class VcardWindow:
 		self.os_info = {0: {'resource': self.contact.resource, 'client': '',
 			'os': ''}}
 		i = 1
-		contact_list = gajim.contacts.get_contact(self.account, self.contact.jid)
+		contact_list = gajim.contacts.get_contacts(self.account, self.contact.jid)
 		if contact_list:
 			for c in contact_list:
 				if c.resource != self.contact.resource:
@@ -367,8 +379,6 @@ class VcardWindow:
 						gobject.idle_add(
 							gajim.connections[self.account].request_os_info, c.jid,
 							c.resource)
-					gajim.connections[self.account].request_last_status_time(c.jid,
-						c.resource)
 					self.os_info[i] = {'resource': c.resource, 'client': '',
 						'os': ''}
 					i += 1
@@ -380,7 +390,12 @@ class VcardWindow:
 		self.fill_status_label()
 
 		if self.gc_contact:
-			gajim.connections[self.account].request_vcard(self.real_jid,
+			# If we know the real jid, remove the resource from vcard request
+			if self.gc_contact.jid:
+				jid = self.gc_contact.jid
+			else:
+				jid = self.real_jid
+			gajim.connections[self.account].request_vcard(jid,
 				self.gc_contact.get_full_jid())
 		else:
 			gajim.connections[self.account].request_vcard(self.contact.jid)
@@ -415,22 +430,6 @@ class ZeroconfVcardWindow:
 		if event.keyval == gtk.keysyms.Escape:
 			self.window.destroy()
 
-	def on_log_history_checkbutton_toggled(self, widget):
-		#log conversation history?
-		oldlog = True
-		no_log_for = gajim.config.get_per('accounts', self.account,
-			'no_log_for').split()
-		if self.contact.jid in no_log_for:
-			oldlog = False
-		log = widget.get_active()
-		if not log and not self.contact.jid in no_log_for:
-			no_log_for.append(self.contact.jid)
-		if log and self.contact.jid in no_log_for:
-			no_log_for.remove(self.contact.jid)
-		if oldlog != log:
-			gajim.config.set_per('accounts', self.account, 'no_log_for',
-				' '.join(no_log_for))
-
 	def on_PHOTO_eventbox_button_press_event(self, widget, event):
 		'''If right-clicked, show popup'''
 		if event.button == 3: # right click
@@ -451,9 +450,11 @@ class ZeroconfVcardWindow:
 			if value and entry_name == 'URL_label':
 				if gtk.pygtk_version >= (2, 10, 0) and gtk.gtk_version >= (2, 10, 0):
 					widget = gtk.LinkButton(value, value)
+					widget.set_alignment(0, 0)
 				else:
 					widget = gtk.Label(value)
 					widget.set_selectable(True)
+					widget.set_alignment(0, 0)
 				table = self.xml.get_widget('personal_info_table')
 				table.attach(widget, 1, 4, 3, 4, yoptions = 0)
 			else:
@@ -464,7 +465,7 @@ class ZeroconfVcardWindow:
 	def fill_status_label(self):
 		if self.xml.get_widget('information_notebook').get_n_pages() < 2:
 			return
-		contact_list = gajim.contacts.get_contact(self.account, self.contact.jid)
+		contact_list = gajim.contacts.get_contacts(self.account, self.contact.jid)
 		# stats holds show and status message
 		stats = ''
 		one = True # Are we adding the first line ?
@@ -499,14 +500,6 @@ class ZeroconfVcardWindow:
 			'</span></b>')
 		self.xml.get_widget('local_jid_label').set_text(self.contact.jid)
 
-		log = True
-		if self.contact.jid in gajim.config.get_per('accounts', self.account,
-			'no_log_for').split(' '):
-			log = False
-		checkbutton = self.xml.get_widget('log_history_checkbutton')
-		checkbutton.set_active(log)
-		checkbutton.connect('toggled', self.on_log_history_checkbutton_toggled)
-		
 		resources = '%s (%s)' % (self.contact.resource, unicode(
 			self.contact.priority))
 		uf_resources = self.contact.resource + _(' resource with priority ')\
@@ -530,7 +523,7 @@ class ZeroconfVcardWindow:
 	def fill_personal_page(self):
 		contact = gajim.connections[gajim.ZEROCONF_ACC_NAME].roster.getItem(self.contact.jid)
 		for key in ('1st', 'last', 'jid', 'email'):
-			if not contact['txt_dict'].has_key(key):
+			if key not in contact['txt_dict']:
 				contact['txt_dict'][key] = ''
 		self.xml.get_widget('first_name_label').set_text(contact['txt_dict']['1st'])
 		self.xml.get_widget('last_name_label').set_text(contact['txt_dict']['last'])
@@ -539,3 +532,5 @@ class ZeroconfVcardWindow:
 
 	def on_close_button_clicked(self, widget):
 		self.window.destroy()
+
+# vim: se ts=3:
